import json
import logging
import mock
import pytest
import requests_mock
import requests

from pubtools._quay import exceptions
from pubtools._quay import push_docker
from .utils.misc import (
    sort_dictionary_sortable_values,
    compare_logs,
    IIBRes,
    mock_manifest_list_requests,
)

MSG_SIGNER_OPERATION_RESULT = [
    [
        {
            "i": 0,
            "msg": {
                "errors": [],
                "manifest_digest": "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                "pub_task_id": "12345",
                "repo": "iib-namespace/new-index-image",
                "request_id": "89cf86e0-8403-46e0-b5ed-5984a635e89e",
                "request_received_time": "2023-10-17T08:08:01.544757",
                "sig_key_id": "37036783",
                "sig_keyname": "testing",
                "signature_type": "container_signature",
                "signed_claim": "claim1",
            },
        },
        {},
    ]
]

# flake8: noqa: E501


@mock.patch("pubtools._quay.push_docker.QuayClient")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_init_verify_target_settings_ok(
    mock_quay_api_client,
    mock_quay_client,
    target_settings,
    container_multiarch_push_item,
    operator_push_item_ok,
):
    hub = mock.MagicMock()
    push_docker_instance = push_docker.PushDocker(
        [container_multiarch_push_item, operator_push_item_ok],
        hub,
        "1",
        "some-target",
        target_settings,
    )
    assert push_docker_instance.push_items == [
        container_multiarch_push_item,
        operator_push_item_ok,
    ]
    assert push_docker_instance.hub == hub
    assert push_docker_instance.task_id == "1"
    assert push_docker_instance.target_name == "some-target"
    assert push_docker_instance.target_settings == target_settings
    assert push_docker_instance.quay_host == "quay.io"
    mock_quay_client.assert_not_called()
    mock_quay_api_client.assert_not_called()

    assert push_docker_instance.dest_quay_client == mock_quay_client.return_value
    assert push_docker_instance.dest_quay_api_client == mock_quay_api_client.return_value
    mock_quay_client.assert_called_once_with("dest-quay-user", "dest-quay-pass", "quay.io")
    mock_quay_api_client.assert_called_once_with("dest-quay-token", "quay.io")


@mock.patch("pubtools._quay.push_docker.QuayClient")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_init_verify_target_settings_missing_item(
    mock_quay_api_client,
    mock_quay_client,
    target_settings,
    container_multiarch_push_item,
    operator_push_item_ok,
):
    hub = mock.MagicMock()
    target_settings.pop("source_quay_user", None)
    with pytest.raises(
        exceptions.InvalidTargetSettings, match="'source_quay_user' must be present.*"
    ):
        push_docker_instance = push_docker.PushDocker(
            [container_multiarch_push_item, operator_push_item_ok],
            hub,
            "1",
            "some-target",
            target_settings,
        )


@mock.patch("pubtools._quay.push_docker.QuayClient")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_init_verify_target_settings_missing_docker_item(
    mock_quay_api_client,
    mock_quay_client,
    target_settings,
    container_multiarch_push_item,
    operator_push_item_ok,
):
    hub = mock.MagicMock()
    target_settings["docker_settings"].pop("umb_urls", None)
    with pytest.raises(exceptions.InvalidTargetSettings, match="'umb_urls' must be present.*"):
        push_docker_instance = push_docker.PushDocker(
            [container_multiarch_push_item, operator_push_item_ok],
            hub,
            "1",
            "some-target",
            target_settings,
        )


@mock.patch("pubtools._quay.push_docker.QuayClient")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_get_container_push_items_ok(
    mock_quay_api_client,
    mock_quay_client,
    target_settings,
    container_multiarch_push_item,
    operator_push_item_ok,
    container_source_push_item,
):
    hub = mock.MagicMock()
    push_docker_instance = push_docker.PushDocker(
        [
            container_multiarch_push_item,
            container_multiarch_push_item,
            operator_push_item_ok,
            container_source_push_item,
        ],
        hub,
        "1",
        "some-target",
        target_settings,
    )
    items = push_docker_instance.get_docker_push_items()
    assert items == [container_multiarch_push_item, container_source_push_item]


@mock.patch("pubtools._quay.push_docker.QuayClient")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_get_container_push_items_errors(
    mock_quay_api_client, mock_quay_client, target_settings, container_push_item_errors
):
    hub = mock.MagicMock()
    push_docker_instance = push_docker.PushDocker(
        [container_push_item_errors], hub, "1", "some-target", target_settings
    )
    with pytest.raises(exceptions.BadPushItem, match=".*contains errors.*"):
        items = push_docker_instance.get_docker_push_items()


@mock.patch("pubtools._quay.push_docker.QuayClient")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_get_container_push_items_no_pull_data(
    mock_quay_api_client,
    mock_quay_client,
    target_settings,
    container_push_item_no_metadata,
):
    hub = mock.MagicMock()
    push_docker_instance = push_docker.PushDocker(
        [container_push_item_no_metadata], hub, "1", "some-target", target_settings
    )
    with pytest.raises(exceptions.BadPushItem, match=".*doesn't contain pull data.*"):
        items = push_docker_instance.get_docker_push_items()


@mock.patch("pubtools._quay.push_docker.QuayClient")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_get_operator_push_items_ok(
    mock_quay_api_client,
    mock_quay_client,
    target_settings,
    operator_push_item_ok,
    operator_push_item_ok2,
    container_push_item_ok,
):
    hub = mock.MagicMock()
    push_docker_instance = push_docker.PushDocker(
        [operator_push_item_ok, operator_push_item_ok2, container_push_item_ok],
        hub,
        "1",
        "some-target",
        target_settings,
    )
    items = push_docker_instance.get_operator_push_items()
    assert items == [operator_push_item_ok, operator_push_item_ok2]


@mock.patch("pubtools._quay.push_docker.QuayClient")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_get_operator_push_item_errors(
    mock_quay_api_client, mock_quay_client, target_settings, operator_push_item_errors
):
    hub = mock.MagicMock()
    push_docker_instance = push_docker.PushDocker(
        [operator_push_item_errors], hub, "1", "some-target", target_settings
    )
    with pytest.raises(exceptions.BadPushItem, match=".*contains errors.*"):
        items = push_docker_instance.get_operator_push_items()


@mock.patch("pubtools._quay.push_docker.QuayClient")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_get_operator_push_item_no_op_type(
    mock_quay_api_client,
    mock_quay_client,
    target_settings,
    operator_push_item_no_op_type,
):
    hub = mock.MagicMock()
    push_docker_instance = push_docker.PushDocker(
        [operator_push_item_no_op_type], hub, "1", "some-target", target_settings
    )
    with pytest.raises(exceptions.BadPushItem, match=".*doesn't contain 'op_type'.*"):
        items = push_docker_instance.get_operator_push_items()


@mock.patch("pubtools._quay.push_docker.QuayClient")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_get_operator_push_item_op_appregistry(
    mock_quay_api_client,
    mock_quay_client,
    target_settings,
    operator_push_item_ok2,
    operator_push_item_appregistry,
):
    hub = mock.MagicMock()
    push_docker_instance = push_docker.PushDocker(
        [operator_push_item_ok2, operator_push_item_appregistry],
        hub,
        "1",
        "some-target",
        target_settings,
    )
    items = push_docker_instance.get_operator_push_items()
    assert items == [operator_push_item_ok2]


@mock.patch("pubtools._quay.push_docker.QuayClient")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_get_operator_push_item_unknown_op_type(
    mock_quay_api_client,
    mock_quay_client,
    target_settings,
    operator_push_item_unknown_op_type,
):
    hub = mock.MagicMock()
    push_docker_instance = push_docker.PushDocker(
        [operator_push_item_unknown_op_type], hub, "1", "some-target", target_settings
    )
    with pytest.raises(exceptions.BadPushItem, match=".*has unknown op_type.*"):
        items = push_docker_instance.get_operator_push_items()


@mock.patch("pubtools._quay.push_docker.QuayClient")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_get_operator_push_item_no_ocp_versions(
    mock_quay_api_client, mock_quay_client, target_settings, operator_push_item_no_ocp
):
    hub = mock.MagicMock()
    push_docker_instance = push_docker.PushDocker(
        [operator_push_item_no_ocp], hub, "1", "some-target", target_settings
    )
    with pytest.raises(exceptions.BadPushItem, match=".*specify 'com.redhat.openshift.versions'.*"):
        items = push_docker_instance.get_operator_push_items()


@mock.patch("pubtools._quay.push_docker.pyxis_get_repo_metadata")
@mock.patch("pubtools._quay.push_docker.QuayClient")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_check_repos_validity_success(
    mock_quay_api_client,
    mock_quay_client,
    mock_get_repo_metadata,
    target_settings,
    container_push_item_correct_repos,
    container_signing_push_item,
    container_push_item_external_repos,
    fake_cert_key_paths,
):
    target_settings["do_repo_deprecation_check"] = True
    mock_get_target_info = mock.MagicMock()
    mock_get_target_info.return_value = {
        "settings": {
            "quay_namespace": "stage_namespace",
            "dest_quay_user": "stage-user",
            "dest_quay_password": "stage-password",
        }
    }
    mock_worker = mock.MagicMock()
    mock_worker.get_target_info = mock_get_target_info
    hub = mock.MagicMock()
    hub.worker = mock_worker

    mock_get_repository_tags = mock.MagicMock()
    mock_get_repository_tags.side_effect = ["repo_data1", "repo_data2", "repo_data3"]
    mock_quay_client.return_value.get_repository_tags = mock_get_repository_tags

    mock_get_repo_metadata.side_effect = [
        {"release_categories": "value2"},
        {"release_categories": "value1"},
        {"release_categories": "value2"},
    ]
    target_settings["propagated_from"] = "target_stage_quay"
    push_docker_instance = push_docker.PushDocker(
        [container_push_item_correct_repos, container_signing_push_item],
        hub,
        "1",
        "some-target",
        target_settings,
    )
    push_docker_instance.check_repos_validity(
        [
            container_push_item_external_repos,
            container_push_item_correct_repos,
            container_signing_push_item,
        ],
        hub,
        target_settings,
    )

    mock_get_target_info.assert_called_once_with("target_stage_quay")
    assert mock_get_repo_metadata.call_count == 3
    mock_get_repo_metadata.call_args_list[0] == mock.call("namespace/repo1")
    mock_get_repo_metadata.call_args_list[1] == mock.call("namespace/repo2")
    mock_get_repo_metadata.call_args_list[2] == mock.call("namespace/repo3")
    assert mock_get_repository_tags.call_count == 3
    mock_get_repository_tags.call_args_list[0] == mock.call("some-namespace/namespace----repo1")
    mock_get_repository_tags.call_args_list[1] == mock.call("some-namespace/namespace----repo2")


@mock.patch("pubtools._quay.push_docker.pyxis_get_repo_metadata")
@mock.patch("pubtools._quay.push_docker.QuayClient")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_check_repos_validity_missing_repo(
    mock_quay_api_client,
    mock_quay_client,
    mock_get_repo_metadata,
    target_settings,
    container_signing_push_item,
):
    target_settings["do_repo_deprecation_check"] = True
    mock_get_target_info = mock.MagicMock()
    mock_get_target_info.return_value = {
        "settings": {
            "quay_namespace": "stage_namespace",
            "dest_quay_user": "stage-user",
            "dest_quay_password": "stage-password",
        }
    }
    mock_worker = mock.MagicMock()
    mock_worker.get_target_info = mock_get_target_info
    hub = mock.MagicMock()
    hub.worker = mock_worker

    mock_get_repository_tags = mock.MagicMock()
    mock_get_repository_tags.side_effect = ["repo_data1", "repo_data2"]
    mock_quay_client.return_value.get_repository_tags = mock_get_repository_tags

    response = mock.MagicMock()
    response.status_code = 404
    mock_get_repo_metadata.side_effect = [
        {"release_categories": "value1"},
        requests.exceptions.HTTPError("missing", response=response),
    ]
    target_settings["propagated_from"] = "target_stage_quay"
    push_docker_instance = push_docker.PushDocker(
        [container_signing_push_item],
        hub,
        "1",
        "some-target",
        target_settings,
    )
    push_docker_instance.check_repos_validity(
        [container_signing_push_item],
        hub,
        target_settings,
    )

    mock_get_target_info.assert_called_once_with("target_stage_quay")
    assert mock_get_repo_metadata.call_count == 2
    mock_get_repo_metadata.call_args_list[0] == mock.call("namespace/repo1")
    mock_get_repo_metadata.call_args_list[1] == mock.call("namespace/repo2")
    assert mock_get_repository_tags.call_count == 2
    mock_get_repository_tags.call_args_list[0] == mock.call("some-namespace/namespace----repo1")
    mock_get_repository_tags.call_args_list[1] == mock.call("some-namespace/namespace----repo2")


@mock.patch("pubtools._quay.push_docker.pyxis_get_repo_metadata")
@mock.patch("pubtools._quay.push_docker.QuayClient")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_check_repos_validity_get_repo_server_error(
    mock_quay_api_client,
    mock_quay_client,
    mock_get_repo_metadata,
    target_settings,
    container_push_item_ok,
    container_signing_push_item,
):
    target_settings["do_repo_deprecation_check"] = True
    mock_get_target_info = mock.MagicMock()
    mock_get_target_info.return_value = {
        "settings": {
            "quay_namespace": "stage_namespace",
            "dest_quay_user": "stage-user",
            "dest_quay_password": "stage-password",
        }
    }
    mock_worker = mock.MagicMock()
    mock_worker.get_target_info = mock_get_target_info
    hub = mock.MagicMock()
    hub.worker = mock_worker

    response = mock.MagicMock()
    response.status_code = 500
    mock_get_repo_metadata.side_effect = [
        {"release_categories": "value1"},
        requests.exceptions.HTTPError("server error", response=response),
    ]
    target_settings["propagated_from"] = "target_stage_quay"
    push_docker_instance = push_docker.PushDocker(
        [container_push_item_ok, container_signing_push_item],
        hub,
        "1",
        "some-target",
        target_settings,
    )
    with pytest.raises(requests.exceptions.HTTPError, match=".*server error.*"):
        push_docker_instance.check_repos_validity(
            [container_push_item_ok, container_signing_push_item],
            hub,
            target_settings,
        )

    mock_get_target_info.assert_called_once_with("target_stage_quay")
    assert mock_get_repo_metadata.call_count == 2
    mock_get_repo_metadata.call_args_list[0] == mock.call("namespace/repo1")
    mock_get_repo_metadata.call_args_list[1] == mock.call("namespace/repo2")


@mock.patch("pubtools._quay.push_docker.pyxis_get_repo_metadata")
@mock.patch("pubtools._quay.push_docker.QuayClient")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_check_repos_validity_deprecated_repo(
    mock_quay_api_client,
    mock_quay_client,
    mock_get_repo_metadata,
    target_settings,
    container_push_item_ok,
    container_signing_push_item,
    fake_cert_key_paths,
):
    target_settings["do_repo_deprecation_check"] = True
    mock_get_target_info = mock.MagicMock()
    mock_get_target_info.return_value = {
        "settings": {
            "quay_namespace": "stage_namespace",
            "dest_quay_user": "stage-user",
            "dest_quay_password": "stage-password",
        }
    }
    mock_worker = mock.MagicMock()
    mock_worker.get_target_info = mock_get_target_info
    hub = mock.MagicMock()
    hub.worker = mock_worker

    mock_get_repo_metadata.side_effect = [
        {"release_categories": "value1"},
        {"release_categories": "Deprecated"},
    ]
    target_settings["propagated_from"] = "target_stage_quay"
    push_docker_instance = push_docker.PushDocker(
        [container_push_item_ok, container_signing_push_item],
        hub,
        "1",
        "some-target",
        target_settings,
    )
    with pytest.raises(exceptions.InvalidRepository, match=".*is deprecated.*"):
        push_docker_instance.check_repos_validity(
            [container_push_item_ok, container_signing_push_item],
            hub,
            target_settings,
        )

    mock_get_target_info.assert_called_once_with("target_stage_quay")
    assert mock_get_repo_metadata.call_count == 2
    mock_get_repo_metadata.call_args_list[0] == mock.call("namespace/repo1")
    mock_get_repo_metadata.call_args_list[1] == mock.call("namespace/repo2")


@mock.patch("pubtools._quay.utils.misc.pyxis_get_repo_metadata")
@mock.patch("pubtools._quay.push_docker.QuayClient")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_check_repos_validity_deprecated_repo_check_disabled(
    mock_quay_api_client,
    mock_quay_client,
    mock_get_repo_metadata,
    target_settings,
    container_push_item_ok,
    container_signing_push_item,
):
    mock_get_target_info = mock.MagicMock()
    mock_get_target_info.return_value = {
        "settings": {
            "quay_namespace": "stage_namespace",
            "dest_quay_user": "stage-user",
            "dest_quay_password": "stage-password",
        }
    }
    mock_worker = mock.MagicMock()
    mock_worker.get_target_info = mock_get_target_info
    hub = mock.MagicMock()
    hub.worker = mock_worker

    mock_get_repo_metadata.return_value = {"release_categories": "Deprecated"}
    target_settings["propagated_from"] = "target_stage_quay"
    push_docker_instance = push_docker.PushDocker(
        [container_push_item_ok, container_signing_push_item],
        hub,
        "1",
        "some-target",
        target_settings,
    )
    push_docker_instance.check_repos_validity(
        [container_push_item_ok, container_signing_push_item],
        hub,
        target_settings,
    )

    mock_get_target_info.assert_called_once_with("target_stage_quay")
    assert mock_get_repo_metadata.call_count == 0


@mock.patch("pubtools._quay.push_docker.pyxis_get_repo_metadata")
@mock.patch("pubtools._quay.push_docker.QuayClient")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_check_repos_validity_missing_stage_repo(
    mock_quay_api_client,
    mock_quay_client,
    mock_get_repo_metadata,
    target_settings,
    container_push_item_ok,
    container_signing_push_item,
    fake_cert_key_paths,
):
    target_settings["do_repo_deprecation_check"] = True
    mock_get_target_info = mock.MagicMock()
    mock_get_target_info.return_value = {
        "settings": {
            "quay_namespace": "stage_namespace",
            "dest_quay_user": "stage-user",
            "dest_quay_password": "stage-password",
        }
    }
    mock_worker = mock.MagicMock()
    mock_worker.get_target_info = mock_get_target_info
    hub = mock.MagicMock()
    hub.worker = mock_worker

    response = mock.MagicMock()
    response.status_code = 404
    mock_get_repository_tags = mock.MagicMock()
    mock_get_repository_tags.side_effect = [
        "repo_data1",
        requests.exceptions.HTTPError("missing", response=response),
    ]
    mock_quay_client.return_value.get_repository_tags = mock_get_repository_tags

    mock_get_repo_metadata.side_effect = [
        {"release_categories": "value1"},
        {"release_categories": "value2"},
    ]
    target_settings["propagated_from"] = "target_stage_quay"
    push_docker_instance = push_docker.PushDocker(
        [container_push_item_ok, container_signing_push_item],
        hub,
        "1",
        "some-target",
        target_settings,
    )
    with pytest.raises(exceptions.InvalidRepository, match=".*doesn't exist on stage.*"):
        push_docker_instance.check_repos_validity(
            [container_push_item_ok, container_signing_push_item],
            hub,
            target_settings,
        )

    mock_get_target_info.assert_called_once_with("target_stage_quay")
    assert mock_get_repo_metadata.call_count == 2
    mock_get_repo_metadata.call_args_list[0] == mock.call("namespace/repo1")
    mock_get_repo_metadata.call_args_list[1] == mock.call("namespace/repo2")
    assert mock_get_repository_tags.call_count == 2
    mock_get_repository_tags.call_args_list[0] == mock.call("some-namespace/namespace----repo1")
    mock_get_repository_tags.call_args_list[1] == mock.call("some-namespace/namespace----repo2")


@mock.patch("pubtools._quay.push_docker.pyxis_get_repo_metadata")
@mock.patch("pubtools._quay.push_docker.QuayClient")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_check_repos_validity_get_stage_repo_server_error(
    mock_quay_api_client,
    mock_quay_client,
    mock_get_repo_metadata,
    target_settings,
    container_push_item_ok,
    container_signing_push_item,
    fake_cert_key_paths,
):
    target_settings["do_repo_deprecation_check"] = True
    mock_get_target_info = mock.MagicMock()
    mock_get_target_info.return_value = {
        "settings": {
            "quay_namespace": "stage_namespace",
            "dest_quay_user": "stage-user",
            "dest_quay_password": "stage-password",
        }
    }
    mock_worker = mock.MagicMock()
    mock_worker.get_target_info = mock_get_target_info
    hub = mock.MagicMock()
    hub.worker = mock_worker

    response = mock.MagicMock()
    response.status_code = 500
    mock_get_repository_tags = mock.MagicMock()
    mock_get_repository_tags.side_effect = [
        "repo_data1",
        requests.exceptions.HTTPError("server error", response=response),
    ]
    mock_quay_client.return_value.get_repository_tags = mock_get_repository_tags

    mock_get_repo_metadata.side_effect = [
        {"release_categories": "value1"},
        {"release_categories": "value2"},
    ]
    target_settings["propagated_from"] = "target_stage_quay"
    push_docker_instance = push_docker.PushDocker(
        [container_push_item_ok, container_signing_push_item],
        hub,
        "1",
        "some-target",
        target_settings,
    )
    with pytest.raises(requests.exceptions.HTTPError, match=".*server error*"):
        push_docker_instance.check_repos_validity(
            [container_push_item_ok, container_signing_push_item],
            hub,
            target_settings,
        )

    mock_get_target_info.assert_called_once_with("target_stage_quay")
    assert mock_get_repo_metadata.call_count == 2
    assert mock_get_repo_metadata.call_args_list[0] == mock.call("namespace/repo1", target_settings)
    assert mock_get_repo_metadata.call_args_list[1] == mock.call("namespace/repo2", target_settings)
    assert mock_get_repository_tags.call_count == 2
    assert mock_get_repository_tags.call_args_list[0] == mock.call(
        "stage_namespace/namespace----repo1"
    )
    assert mock_get_repository_tags.call_args_list[1] == mock.call(
        "stage_namespace/namespace----repo2"
    )


@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_generate_backup_mapping(
    mock_quay_api_client,
    target_settings,
    container_multiarch_push_item,
    container_signing_push_item,
    container_push_item_ok,
    src_manifest_list,
    v2s1_manifest,
    fixture_run_in_parallel,
):
    hub = mock.MagicMock()

    push_docker_instance = push_docker.PushDocker(
        [
            container_multiarch_push_item,
            container_signing_push_item,
            container_push_item_ok,
        ],
        hub,
        "1",
        "some-target",
        target_settings,
    )
    with requests_mock.Mocker() as m:
        mock_manifest_list_requests(
            m,
            "https://quay.io/v2/some-namespace/test-repo/manifests/latest-test-tag",
            src_manifest_list,
            v2s1_manifest,
        )
        m.get(
            "https://quay.io/v2/some-namespace/test-repo/manifests/1.0",
            text="Not Found",
            status_code=404,
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.list.v2+json"},
        )
        m.get(
            "https://quay.io/v2/some-namespace/target----repo/manifests/latest-test-tag",
            text="Not Found",
            status_code=404,
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.list.v2+json"},
        )
        mock_manifest_list_requests(
            m,
            "https://quay.io/v2/some-namespace/target----repo1/manifests/tag1",
            src_manifest_list,
            v2s1_manifest,
        )
        m.get(
            "https://quay.io/v2/some-namespace/target----repo1/manifests/tag2",
            text="Not Found",
            status_code=404,
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.list.v2+json"},
        )
        m.get(
            "https://quay.io/v2/some-namespace/target----repo1/manifests/tag2",
            text="Not Found",
            status_code=404,
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.list.v2+json"},
        )
        mock_manifest_list_requests(
            m,
            "https://quay.io/v2/some-namespace/target----repo2/manifests/tag3",
            src_manifest_list,
            v2s1_manifest,
        )
        backup_tags, rollback_tags = push_docker_instance.generate_backup_mapping(
            [
                container_multiarch_push_item,
                container_signing_push_item,
                container_push_item_ok,
            ]
        )

    assert backup_tags == {
        push_docker.PushDocker.ImageData(
            repo="some-namespace/target----repo1",
            tag="tag1",
            v2s2_digest="",
            v2s1_digest="",
            v2list_digest="manifest_list_digest",
        ): (src_manifest_list, ""),
        push_docker.PushDocker.ImageData(
            repo="some-namespace/target----repo2",
            tag="tag3",
            v2s2_digest="",
            v2s1_digest="",
            v2list_digest="manifest_list_digest",
        ): (src_manifest_list, ""),
        push_docker.PushDocker.ImageData(
            repo="some-namespace/test-repo",
            tag="latest-test-tag",
            v2s2_digest="",
            v2s1_digest="",
            v2list_digest="manifest_list_digest",
        ): (src_manifest_list, ""),
    }
    assert rollback_tags == [
        push_docker.PushDocker.ImageData(
            repo="some-namespace/target----repo",
            tag="latest-test-tag",
            v2s2_digest=None,
            v2s1_digest=None,
            v2list_digest=None,
        ),
        push_docker.PushDocker.ImageData(
            repo="some-namespace/target----repo1",
            tag="tag2",
            v2s2_digest=None,
            v2s1_digest=None,
            v2list_digest=None,
        ),
        push_docker.PushDocker.ImageData(
            repo="some-namespace/test-repo",
            tag="1.0",
            v2s2_digest=None,
            v2s1_digest=None,
            v2list_digest=None,
        ),
    ]


@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_generate_backup_mapping_server_error(
    mock_quay_api_client,
    target_settings,
    container_multiarch_push_item,
    container_signing_push_item,
    src_manifest_list,
    v2s1_manifest,
):
    hub = mock.MagicMock()

    with requests_mock.Mocker() as m:
        m.get(
            "https://quay.io/v2/some-namespace/target----repo/manifests/latest-test-tag",
            [{"status_code": 500, "reason": "server error"}],
        )
        mock_manifest_list_requests(
            m,
            "https://quay.io/v2/some-namespace/target----repo1/manifests/tag1",
            src_manifest_list,
            v2s1_manifest,
        )
        mock_manifest_list_requests(
            m,
            "https://quay.io/v2/some-namespace/target----repo1/manifests/tag2",
            src_manifest_list,
            v2s1_manifest,
        )
        mock_manifest_list_requests(
            m,
            "https://quay.io/v2/some-namespace/target----repo2/manifests/tag3",
            src_manifest_list,
            v2s1_manifest,
        )
        mock_manifest_list_requests(
            m,
            "https://quay.io/v2/some-namespace/target----repo1/manifests/some-other-tag",
            src_manifest_list,
            v2s1_manifest,
        )
        mock_manifest_list_requests(
            m,
            "https://quay.io/v2/some-namespace/target----repo/manifests/some-other-tag",
            src_manifest_list,
            v2s1_manifest,
        )
        push_docker_instance = push_docker.PushDocker(
            [container_multiarch_push_item],
            hub,
            "1",
            "some-target",
            target_settings,
        )
        with pytest.raises(requests.exceptions.HTTPError, match=".*server error*"):
            backup_tags, rollback_tags = push_docker_instance.generate_backup_mapping(
                [container_multiarch_push_item, container_signing_push_item]
            )


@mock.patch("pubtools._quay.push_docker.QuayClient")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_rollback(
    mock_quay_api_client,
    mock_quay_client,
    target_settings,
    container_multiarch_push_item,
    container_signing_push_item,
):
    hub = mock.MagicMock()
    mock_upload_manifest = mock.MagicMock()
    mock_quay_client.return_value.upload_manifest = mock_upload_manifest
    mock_delete_tag = mock.MagicMock()
    mock_quay_api_client.return_value.delete_tag = mock_delete_tag

    backup_tags = {
        push_docker.PushDocker.ImageData(
            repo="some-namespace/target----repo1",
            tag="1",
            v2list_digest=None,
            v2s2_digest=None,
            v2s1_digest=None,
        ): "some-manifest-list",
        push_docker.PushDocker.ImageData(
            repo="some-namespace/target----repo2",
            tag="2",
            v2list_digest=None,
            v2s2_digest=None,
            v2s1_digest=None,
        ): "other-manifest-list",
    }
    rollback_tags = [
        push_docker.PushDocker.ImageData(
            repo="some-namespace/target----repo3",
            tag="3",
            v2list_digest=None,
            v2s2_digest=None,
            v2s1_digest=None,
        ),
        push_docker.PushDocker.ImageData(
            repo="some-namespace/target----repo4",
            tag="4",
            v2list_digest=None,
            v2s2_digest=None,
            v2s1_digest=None,
        ),
    ]
    push_docker_instance = push_docker.PushDocker(
        [container_multiarch_push_item, container_signing_push_item],
        hub,
        "1",
        "some-target",
        target_settings,
    )
    push_docker_instance.rollback(backup_tags, rollback_tags)

    assert mock_upload_manifest.call_count == 2
    assert mock_upload_manifest.call_args_list[0] == mock.call(
        "some-manifest-list", "quay.io/some-namespace/target----repo1:1"
    )
    assert mock_upload_manifest.call_args_list[1] == mock.call(
        "other-manifest-list", "quay.io/some-namespace/target----repo2:2"
    )
    assert mock_delete_tag.call_count == 2
    assert mock_delete_tag.call_args_list[0] == mock.call("some-namespace/target----repo3", "3")
    assert mock_delete_tag.call_args_list[1] == mock.call("some-namespace/target----repo4", "4")


@mock.patch("pubtools._quay.push_docker.QuayClient")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_rollback_deleted_tag_not_found(
    mock_quay_api_client,
    mock_quay_client,
    target_settings,
    container_multiarch_push_item,
    container_signing_push_item,
):
    hub = mock.MagicMock()
    mock_delete_tag = mock.MagicMock()
    response = mock.MagicMock()
    response.status_code = 404
    mock_delete_tag.side_effect = [
        requests.exceptions.HTTPError("not found", response=response),
        None,
    ]
    mock_quay_api_client.return_value.delete_tag = mock_delete_tag

    backup_tags = {}
    rollback_tags = [
        push_docker.PushDocker.ImageData(
            repo="some-namespace/target----repo3",
            tag="3",
            v2list_digest=None,
            v2s2_digest=None,
            v2s1_digest=None,
        ),
        push_docker.PushDocker.ImageData(
            repo="some-namespace/target----repo4",
            tag="4",
            v2list_digest=None,
            v2s2_digest=None,
            v2s1_digest=None,
        ),
    ]
    push_docker_instance = push_docker.PushDocker(
        [container_multiarch_push_item, container_signing_push_item],
        hub,
        "1",
        "some-target",
        target_settings,
    )
    push_docker_instance.rollback(backup_tags, rollback_tags)

    assert mock_delete_tag.call_count == 2
    assert mock_delete_tag.call_args_list[0] == mock.call("some-namespace/target----repo3", "3")
    assert mock_delete_tag.call_args_list[1] == mock.call("some-namespace/target----repo4", "4")


@mock.patch("pubtools._quay.push_docker.QuayClient")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_rollback_delete_tag_server_error(
    mock_quay_api_client,
    mock_quay_client,
    target_settings,
    container_multiarch_push_item,
    container_signing_push_item,
):
    hub = mock.MagicMock()
    mock_delete_tag = mock.MagicMock()
    response = mock.MagicMock()
    response.status_code = 500
    mock_delete_tag.side_effect = [
        requests.exceptions.HTTPError("server error", response=response),
        None,
    ]
    mock_quay_api_client.return_value.delete_tag = mock_delete_tag

    backup_tags = {}
    rollback_tags = [
        push_docker.PushDocker.ImageData(
            repo="some-namespace/target----repo3",
            tag="3",
            v2list_digest=None,
            v2s2_digest=None,
            v2s1_digest=None,
        ),
        push_docker.PushDocker.ImageData(
            repo="some-namespace/target----repo4",
            tag="4",
            v2list_digest=None,
            v2s2_digest=None,
            v2s1_digest=None,
        ),
    ]
    push_docker_instance = push_docker.PushDocker(
        [container_multiarch_push_item, container_signing_push_item],
        hub,
        "1",
        "some-target",
        target_settings,
    )
    with pytest.raises(requests.exceptions.HTTPError, match=".*server error.*"):
        push_docker_instance.rollback(backup_tags, rollback_tags)

    mock_delete_tag.assert_called_once_with("some-namespace/target----repo3", "3")


@mock.patch("pubtools._quay.push_docker.set_aws_kms_environment_variables")
@mock.patch("pubtools._quay.push_docker.SecurityManifestPusher")
@mock.patch("pubtools._quay.push_docker.timestamp")
@mock.patch("pubtools._quay.push_docker.PushDocker.rollback")
@mock.patch("pubtools._quay.push_docker.OperatorPusher")
@mock.patch("pubtools._quay.push_docker.ContainerImagePusher")
@mock.patch("pubtools._quay.push_docker.PushDocker.generate_backup_mapping")
@mock.patch("pubtools._quay.push_docker.PushDocker.check_repos_validity")
@mock.patch("pubtools._quay.push_docker.PushDocker.get_operator_push_items")
@mock.patch("pubtools._quay.push_docker.PushDocker.get_docker_push_items")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_push_docker_full_success(
    mock_quay_api_client,
    mock_get_docker_push_items,
    mock_get_operator_push_items,
    mock_check_repos_validity,
    mock_generate_backup_mapping,
    mock_container_image_pusher,
    mock_operator_pusher,
    mock_rollback,
    mock_timestamp,
    mock_security_manifest_pusher,
    mock_set_aws_kms_environment_variables,
    target_settings,
    container_multiarch_push_item,
    container_push_item_external_repos,
    container_push_item_ok,
    operator_push_item_ok,
    fake_cert_key_paths,
    signer_wrapper_entry_point,
    signer_wrapper_run_entry_point,
    v2s1_manifest,
    dest_manifest_list,
):
    hub = mock.MagicMock()
    mock_push_container_images = mock.MagicMock()
    mock_container_image_pusher.return_value.push_container_images = mock_push_container_images

    mock_push_index_images = mock.MagicMock()
    mock_operator_pusher.return_value.push_index_images = mock_push_index_images

    mock_get_signatures_from_pyxis = mock.MagicMock(
        return_value=(
            [
                {
                    "manifest_digest": "some-digest",
                    "repository": "orig-ns/some-repo",
                    "reference": "registry/orig-ns/some-repo:sometag",
                    "_id": "signature-id-1",
                }
            ]
        )
    )
    iib_res = IIBRes(
        "registry.com/namespace/index-image@sha256:v4.5",
        "registry.com/namespace/iib@sha256:a1a1a1",
        ["v4.5-1"],
    )
    mock_build_index_images = mock.Mock()
    mock_operator_pusher.return_value.build_index_images = mock_build_index_images
    mock_build_index_images.return_value = {
        "v4.5": {"iib_result": iib_res, "signing_keys": ["sigkey"], "destination_tags": ["v4.5"]}
    }

    mock_push_index_images = mock.MagicMock()
    mock_operator_pusher.return_value.push_index_images = mock_push_index_images

    # mock_operator_signature_handler.return_value.sign_operator_images = mock_sign_operator_images
    mock_timestamp.return_value = "timestamp"

    mock_get_docker_push_items.return_value = [container_multiarch_push_item]
    mock_get_operator_push_items.return_value = [operator_push_item_ok]
    mock_generate_backup_mapping.side_effect = [
        (
            {
                push_docker.PushDocker.ImageData(
                    "some-ns/orig-ns----some-repo", "sometag", "some-digest", None, "some-digest"
                ): ({"digest": "some-digest"}, "amd64"),
                push_docker.PushDocker.ImageData(
                    "some-ns/orig-ns----some-repo", "sometag", None, "some-digest", "some-digest"
                ): ({"digest": "some-digest"}, "amd64"),
                push_docker.PushDocker.ImageData(
                    "some-ns/orig-ns----some-repo",
                    "sometag2",
                    "some-digest-2",
                    None,
                    "some-digest-2",
                ): ({"manifests": [{"digest": "some-digest"}]}, ""),
            },
            ["item1", "item2"],
        ),
        (
            {
                push_docker.PushDocker.ImageData(
                    "some-ns/orig-ns----some-repo",
                    "sometag",
                    "some-new-digest",
                    None,
                    "some-digest",
                ): ({"digest": "some-digest-new"}, "amd64"),
                push_docker.PushDocker.ImageData(
                    "some-ns/orig-ns----some-repo",
                    "sometag",
                    None,
                    "some-digest-new",
                    "some-digest",
                ): ({"digest": "some-digest-new"}, "amd64"),
                push_docker.PushDocker.ImageData(
                    "some-ns/orig-ns----some-repo",
                    "sometag2",
                    "some-digest-2",
                    None,
                    "some-digest-2",
                ): ({"manifests": [{"digest": "some-digest-new"}]}, ""),
            },
            ["item1", "item2"],
        ),
    ]
    # iib_result = mock.MagicMock(internal_index_image_copy_resolved="registry/ns/iib@digest")

    signer_wrapper_entry_point.return_value = {
        "signer_result": {
            "status": "ok",
        },
        "operation": {
            "references": ["some-registry.com/iib-namespace/new-index-image:8"],
            "manifests": [
                "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6"
            ],
        },
        "operation_results": MSG_SIGNER_OPERATION_RESULT,
        "signing_key": "sig-key",
    }
    with requests_mock.Mocker() as m:
        m.get(
            "https://quay.io/v2/src/repo/manifests/1",
            json=v2s1_manifest,
            headers={
                "Content-Type": "application/vnd.docker.distribution.manifest.v2+json",
                "docker-content-digest": "sha256:5555555555",
            },
        )
        m.get(
            "https://quay.io/v2/some-namespace/target----repo/tags/list",
            json={"name": "target-repo", "tags": ["latest-test-tag"]},
        )
        m.get(
            "https://quay.io/v2/some-namespace/target----repo/manifests/latest-test-tag",
            text=json.dumps(v2s1_manifest, sort_keys=True),
            headers={
                "Content-Type": "application/vnd.docker.distribution.manifest.v1+json",
                "docker-content-digest": "sha256:5555555555",
            },
            request_headers={"Accept": "application/vnd.docker.distribution.manifest.v1+json"},
        )
        m.get(
            "https://quay.io/v2/namespace/iib/manifests/v4.5-1",
            json=dest_manifest_list,
            headers={
                "Content-Type": "application/vnd.docker.distribution.manifest.list.v2+json",
                "docker-content-digest": "sha256:5555555555",
            },
        )
        # call to untag old signatures
        m.get(
            "https://quay.io/v2/some-namespace/orig-ns----some-repo/manifests/some-digest",
            json=dest_manifest_list,
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.list.v2+json"},
        )

        push_docker_instance = push_docker.PushDocker(
            [container_multiarch_push_item, container_push_item_ok, operator_push_item_ok],
            hub,
            "1",
            "some-target",
            target_settings,
        )
        push_docker_instance.run()

    mock_get_docker_push_items.assert_called_once_with()
    mock_check_repos_validity.assert_called_once_with(
        [container_multiarch_push_item], hub, target_settings
    )
    mock_generate_backup_mapping.assert_has_calls(
        [
            mock.call([container_multiarch_push_item], all_arches=True),
            mock.call([container_multiarch_push_item], all_arches=True),
        ]
    )
    mock_container_image_pusher.assert_called_once_with(
        [container_multiarch_push_item], target_settings
    )
    mock_push_container_images.assert_called_once_with()
    mock_set_aws_kms_environment_variables.assert_has_calls(
        [
            mock.call(target_settings, "cosign_signer"),
            mock.call(target_settings, "security_manifest_signer"),
        ]
    )
    mock_security_manifest_pusher.assert_called_once_with(
        [container_multiarch_push_item], target_settings
    )
    mock_security_manifest_pusher.return_value.push_security_manifests.assert_called_once_with()
    mock_operator_pusher.assert_called_once_with([operator_push_item_ok], "1", target_settings)
    mock_build_index_images.assert_called_once_with()
    mock_push_index_images.assert_called_once_with(
        {"v4.5": {"iib_result": iib_res, "signing_keys": ["sigkey"], "destination_tags": ["v4.5"]}},
        "timestamp",
    )
    mock_rollback.assert_not_called()


@mock.patch("pubtools._quay.push_docker.SecurityManifestPusher")
@mock.patch("pubtools._quay.push_docker.timestamp")
@mock.patch("pubtools._quay.push_docker.PushDocker.rollback")
@mock.patch("pubtools._quay.push_docker.OperatorPusher")
@mock.patch("pubtools._quay.push_docker.ContainerImagePusher")
@mock.patch("pubtools._quay.push_docker.PushDocker.generate_backup_mapping")
@mock.patch("pubtools._quay.push_docker.PushDocker.check_repos_validity")
@mock.patch("pubtools._quay.push_docker.PushDocker.get_operator_push_items")
@mock.patch("pubtools._quay.push_docker.PushDocker.get_docker_push_items")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_push_docker_full_prerelease(
    mock_quay_api_client,
    mock_get_docker_push_items,
    mock_get_operator_push_items,
    mock_check_repos_validity,
    mock_generate_backup_mapping,
    mock_container_image_pusher,
    mock_operator_pusher,
    mock_rollback,
    mock_timestamp,
    mock_security_manifest_pusher,
    target_settings,
    container_multiarch_pre_release_push_item,
    container_push_item_external_repos,
    signer_wrapper_entry_point,
    signer_wrapper_run_entry_point,
    operator_push_item_ok,
    fake_cert_key_paths,
    v2s1_manifest,
    dest_manifest_list,
):
    hub = mock.MagicMock()
    mock_push_container_images = mock.MagicMock()
    mock_container_image_pusher.return_value.push_container_images = mock_push_container_images

    mock_push_index_images = mock.MagicMock()
    mock_operator_pusher.return_value.push_index_images = mock_push_index_images

    mock_get_signatures_from_pyxis = mock.MagicMock(
        return_value=(
            [
                {
                    "manifest_digest": "some-digest",
                    "repository": "orig-ns/some-repo",
                    "reference": "registry/orig-ns/some-repo:sometag",
                    "_id": "signature-id-1",
                }
            ]
        )
    )
    iib_result = mock.MagicMock(internal_index_image_copy_resolved="registry/ns/iib@digest")
    iib_res = IIBRes(
        "registry.com/namespace/index-image@sha256:v4.5",
        "registry.com/namespace/iib@sha256:a1a1a1",
        ["v4.5-1"],
    )
    mock_build_index_images = mock.MagicMock()
    mock_operator_pusher.return_value.build_index_images = mock_build_index_images
    mock_build_index_images.return_value = {
        "v4.5": {"iib_result": iib_res, "signing_keys": ["sigkey"], "destination_tags": ["v4.5"]}
    }

    mock_push_index_images = mock.MagicMock()
    mock_operator_pusher.return_value.push_index_images = mock_push_index_images

    mock_timestamp.return_value = "timestamp"

    mock_get_docker_push_items.return_value = [container_multiarch_pre_release_push_item]
    mock_get_operator_push_items.return_value = [operator_push_item_ok]
    mock_generate_backup_mapping.side_effect = [
        (
            {
                push_docker.PushDocker.ImageData(
                    "some-ns/orig-ns----some-repo",
                    "sometag",
                    "some-digest-list",
                    "some-digest-sch2",
                    "some-digest-sch1",
                ): ({"digest": "some-digest"}, "amd64"),
                push_docker.PushDocker.ImageData(
                    "some-ns/orig-ns----some-repo", "sometag2", None, "some-digest", None
                ): ({"manifests": [{"digest": "some-digest"}]}, ""),
            },
            ["item1", "item2"],
        ),
        (
            {
                push_docker.PushDocker.ImageData(
                    "some-ns/orig-ns----some-repo",
                    "sometag",
                    "some-digest-list",
                    "some-digest-sch2",
                    "some-digest-sch1-new",
                ): ({"digest": "some-digest"}, "amd64"),
                push_docker.PushDocker.ImageData(
                    "some-ns/orig-ns----some-repo", "sometag2", None, "some-digest", None
                ): ({"manifests": [{"digest": "some-digest"}]}, ""),
            },
            ["item1", "item2"],
        ),
    ]

    signer_wrapper_entry_point.return_value = {
        "signer_result": {
            "status": "ok",
        },
        "operation": {
            "references": ["some-registry.com/iib-namespace/new-index-image:8"],
            "manifests": [
                "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6"
            ],
        },
        "operation_results": MSG_SIGNER_OPERATION_RESULT,
        "signing_key": "sig-key",
    }
    with requests_mock.Mocker() as m:
        m.get(
            "https://quay.io/v2/src/repo/manifests/1",
            json=v2s1_manifest,
            headers={
                "Content-Type": "application/vnd.docker.distribution.manifest.v2+json",
                "docker-content-digest": "sha256:5555555555",
            },
        )
        m.get(
            "https://quay.io/v2/some-namespace/target----repo/tags/list",
            json={"name": "target-repo", "tags": ["latest-test-tag"]},
        )
        m.get(
            "https://quay.io/v2/some-namespace/target----repo/manifests/latest-test-tag",
            text=json.dumps(v2s1_manifest, sort_keys=True),
            headers={
                "Content-Type": "application/vnd.docker.distribution.manifest.v1+json",
                "docker-content-digest": "sha256:5555555555",
            },
            request_headers={"Accept": "application/vnd.docker.distribution.manifest.v1+json"},
        )
        m.get(
            "https://quay.io/v2/namespace/iib/manifests/v4.5-1",
            json=dest_manifest_list,
            headers={
                "Content-Type": "application/vnd.docker.distribution.manifest.list.v2+json",
                "docker-content-digest": "sha256:5555555555",
            },
        )
        # call for untagging old signatures
        m.get(
            "https://quay.io/v2/some-namespace/orig-ns----some-repo/manifests/some-digest-sch1",
            json=dest_manifest_list,
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.list.v2+json"},
        )
        m.get(
            "https://quay.io/v2/some-namespace/orig-ns----some-repo/manifests/some-digest-sch2",
            json=dest_manifest_list,
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.list.v2+json"},
        )
        push_docker_instance = push_docker.PushDocker(
            [container_multiarch_pre_release_push_item, operator_push_item_ok],
            hub,
            "1",
            "some-target",
            target_settings,
        )
        push_docker_instance.run()

    mock_get_docker_push_items.assert_called_once_with()
    mock_check_repos_validity.assert_called_once_with(
        [container_multiarch_pre_release_push_item], hub, target_settings
    )
    mock_generate_backup_mapping.assert_has_calls(
        [
            mock.call([container_multiarch_pre_release_push_item], all_arches=True),
            mock.call([container_multiarch_pre_release_push_item], all_arches=True),
        ]
    )
    mock_container_image_pusher.assert_called_once_with(
        [container_multiarch_pre_release_push_item], target_settings
    )
    mock_push_container_images.assert_called_once_with()
    mock_security_manifest_pusher.assert_called_once_with(
        [container_multiarch_pre_release_push_item], target_settings
    )
    mock_security_manifest_pusher.return_value.push_security_manifests.assert_called_once_with()
    mock_operator_pusher.assert_called_once_with([operator_push_item_ok], "1", target_settings)
    mock_build_index_images.assert_called_once_with()
    mock_push_index_images.assert_called_once_with(
        {"v4.5": {"iib_result": iib_res, "signing_keys": ["sigkey"], "destination_tags": ["v4.5"]}},
        "timestamp",
    )
    mock_rollback.assert_not_called()


@mock.patch("pubtools._quay.push_docker.SecurityManifestPusher")
@mock.patch("pubtools._quay.push_docker.timestamp")
@mock.patch("pubtools._quay.push_docker.PushDocker.rollback")
@mock.patch("pubtools._quay.push_docker.OperatorPusher")
@mock.patch("pubtools._quay.push_docker.ContainerImagePusher")
@mock.patch("pubtools._quay.push_docker.PushDocker.generate_backup_mapping")
@mock.patch("pubtools._quay.push_docker.PushDocker.check_repos_validity")
@mock.patch("pubtools._quay.push_docker.PushDocker.get_operator_push_items")
@mock.patch("pubtools._quay.push_docker.PushDocker.get_docker_push_items")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_push_docker_full_no_v2sch2(
    mock_quay_api_client,
    mock_get_docker_push_items,
    mock_get_operator_push_items,
    mock_check_repos_validity,
    mock_generate_backup_mapping,
    mock_container_image_pusher,
    mock_operator_pusher,
    mock_rollback,
    mock_timestamp,
    mock_security_manifest_pusher,
    target_settings,
    container_multiarch_push_item,
    container_push_item_external_repos,
    operator_push_item_ok,
    dest_manifest_list,
    fake_cert_key_paths,
    signer_wrapper_entry_point,
    signer_wrapper_run_entry_point,
    v2s1_manifest,
):
    hub = mock.MagicMock()
    mock_push_container_images = mock.MagicMock()
    mock_container_image_pusher.return_value.push_container_images = mock_push_container_images

    mock_push_index_images = mock.MagicMock()
    mock_operator_pusher.return_value.push_index_images = mock_push_index_images

    mock_get_signatures_from_pyxis = mock.MagicMock(
        return_value=(
            [
                {
                    "manifest_digest": "some-digest",
                    "repository": "orig-ns/some-repo",
                    "reference": "registry/orig-ns/some-repo:sometag",
                    "_id": "signature-id-1",
                }
            ]
        )
    )
    mock_build_index_images = mock.MagicMock()
    mock_operator_pusher.return_value.build_index_images = mock_build_index_images

    mock_push_index_images = mock.MagicMock()
    mock_operator_pusher.return_value.push_index_images = mock_push_index_images

    mock_timestamp.return_value = "timestamp"

    mock_get_docker_push_items.return_value = [container_multiarch_push_item]
    mock_get_operator_push_items.return_value = [operator_push_item_ok]
    mock_generate_backup_mapping.side_effect = [
        (
            {
                push_docker.PushDocker.ImageData(
                    "some-ns/orig-ns----some-repo", "sometag", None, None, None
                ): ({"digest": "some-digest"}, "amd64"),
                push_docker.PushDocker.ImageData(
                    "some-ns/orig-ns----some-repo", "sometag2", None, None, None
                ): ({"manifests": [{"digest": "some-digest"}]}, ""),
            },
            ["item1", "item2"],
        ),
        (
            {
                push_docker.PushDocker.ImageData(
                    "some-ns/orig-ns----some-repo", "sometag", None, None, None
                ): ({"digest": "some-digest-new"}, "amd64"),
                push_docker.PushDocker.ImageData(
                    "some-ns/orig-ns----some-repo", "sometag2", None, None, None
                ): ({"manifests": [{"digest": "some-digest-new"}]}, ""),
            },
            ["item1", "item2"],
        ),
    ]
    iib_res = IIBRes(
        "registry.com/namespace/index-image@sha256:v4.5",
        "registry.com/namespace/iib@sha256:a1a1a1",
        ["v4.5-1"],
    )
    mock_build_index_images.return_value = {
        "v4.5": {"iib_result": iib_res, "signing_keys": [], "destination_tags": ["v4.5"]}
    }

    signer_wrapper_entry_point.return_value = {
        "signer_result": {
            "status": "ok",
        },
        "operation": {
            "references": ["some-registry.com/iib-namespace/new-index-image:8"],
            "manifests": [
                "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6"
            ],
        },
        "operation_results": MSG_SIGNER_OPERATION_RESULT,
        "signing_key": "sig-key",
    }
    with requests_mock.Mocker() as m:
        m.get(
            "https://quay.io/v2/src/repo/manifests/1",
            json=v2s1_manifest,
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.v2+json"},
        )
        m.get(
            "https://quay.io/v2/some-namespace/target----repo/tags/list",
            json={"name": "target-repo", "tags": ["latest-test-tag"]},
        )
        m.get(
            "https://quay.io/v2/some-namespace/target----repo/manifests/latest-test-tag",
            text=json.dumps(v2s1_manifest, sort_keys=True),
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.v1+json"},
            request_headers={"Accept": "application/vnd.docker.distribution.manifest.v1+json"},
        )
        m.get(
            "https://quay.io/v2/namespace/iib/manifests/v4.5-1",
            json=dest_manifest_list,
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.list.v2+json"},
        )
        push_docker_instance = push_docker.PushDocker(
            [container_multiarch_push_item, operator_push_item_ok],
            hub,
            "1",
            "some-target",
            target_settings,
        )
        push_docker_instance.run()

    mock_get_docker_push_items.assert_called_once_with()
    mock_check_repos_validity.assert_called_once_with(
        [container_multiarch_push_item], hub, target_settings
    )
    mock_generate_backup_mapping.assert_has_calls(
        [
            mock.call([container_multiarch_push_item], all_arches=True),
            mock.call([container_multiarch_push_item], all_arches=True),
        ]
    )
    mock_container_image_pusher.assert_called_once_with(
        [container_multiarch_push_item], target_settings
    )
    mock_push_container_images.assert_called_once_with()
    mock_security_manifest_pusher.assert_called_once_with(
        [container_multiarch_push_item], target_settings
    )
    mock_security_manifest_pusher.return_value.push_security_manifests.assert_called_once_with()
    mock_operator_pusher.assert_called_once_with([operator_push_item_ok], "1", target_settings)
    mock_build_index_images.assert_called_once_with()
    mock_push_index_images.assert_called_once_with(
        {"v4.5": {"iib_result": iib_res, "signing_keys": [], "destination_tags": ["v4.5"]}},
        "timestamp",
    )
    mock_rollback.assert_not_called()


@mock.patch("pubtools._quay.push_docker.SecurityManifestPusher")
@mock.patch("pubtools._quay.push_docker.timestamp")
@mock.patch("pubtools._quay.push_docker.PushDocker.rollback")
@mock.patch("pubtools._quay.push_docker.OperatorPusher")
@mock.patch("pubtools._quay.push_docker.ContainerImagePusher")
@mock.patch("pubtools._quay.push_docker.PushDocker.generate_backup_mapping")
@mock.patch("pubtools._quay.push_docker.PushDocker.check_repos_validity")
@mock.patch("pubtools._quay.push_docker.PushDocker.get_operator_push_items")
@mock.patch("pubtools._quay.push_docker.PushDocker.get_docker_push_items")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_push_docker_full_success_repush(
    mock_quay_api_client,
    mock_get_docker_push_items,
    mock_get_operator_push_items,
    mock_check_repos_validity,
    mock_generate_backup_mapping,
    mock_container_image_pusher,
    mock_operator_pusher,
    mock_rollback,
    mock_timestamp,
    mock_security_manifest_pusher,
    target_settings,
    container_multiarch_push_item,
    container_push_item_external_repos,
    operator_push_item_ok,
    fake_cert_key_paths,
    v2s1_manifest,
    dest_manifest_list,
    signer_wrapper_entry_point,
    signer_wrapper_run_entry_point,
):
    hub = mock.MagicMock()
    mock_push_container_images = mock.MagicMock()
    mock_container_image_pusher.return_value.push_container_images = mock_push_container_images
    mock_sign_container_images = mock.MagicMock(return_value=[])
    mock_sign_container_images_new_digests = mock.MagicMock(return_value=[])
    mock_build_index_images = mock.MagicMock()
    mock_operator_pusher.return_value.build_index_images = mock_build_index_images

    mock_push_index_images = mock.MagicMock()
    mock_operator_pusher.return_value.push_index_images = mock_push_index_images
    mock_get_existing_index_images = mock.MagicMock(
        return_value=[("somedigest", "sometag", "somerepo")]
    )
    mock_timestamp.return_value = "timestamp"
    mock_operator_pusher.return_value.get_existing_index_images = mock_get_existing_index_images

    mock_get_docker_push_items.return_value = [
        container_multiarch_push_item,
        container_push_item_external_repos,
    ]
    mock_get_operator_push_items.return_value = [operator_push_item_ok]
    mock_generate_backup_mapping.side_effect = [
        (
            {
                push_docker.PushDocker.ImageData(
                    "some-ns/orig-ns----somerepo", "sometag", None, None, None
                ): ({"digest": "some-digest"}, "amd64"),
                push_docker.PushDocker.ImageData(
                    "some-ns/orig-ns----somerepo", "sometag2", None, None, None
                ): ({"manifests": [{"digest": "some-digest"}]}, ""),
            },
            ["item1", "item2"],
        ),
        (
            {
                push_docker.PushDocker.ImageData(
                    "some-ns/orig-ns----somerepo", "sometag", None, None, None
                ): ({"digest": "some-digest-new"}, "amd64"),
                push_docker.PushDocker.ImageData(
                    "some-ns/orig-ns----somerepo", "sometag2", None, None, None
                ): ({"manifests": [{"digest": "some-digest-new"}]}, ""),
            },
            ["item1", "item2"],
        ),
    ]
    iib_res = IIBRes(
        "registry.com/namespace/index-image@sha256:v4.5",
        "registry.com/namespace/iib@sha256:a1a1a1",
        ["v4.5-1"],
    )
    mock_build_index_images.return_value = {
        "v4.5": {"iib_result": iib_res, "signing_keys": [], "destination_tags": ["v4.5"]}
    }
    signer_wrapper_entry_point.return_value = {
        "signer_result": {
            "status": "ok",
        },
        "operation": {
            "references": ["some-registry.com/iib-namespace/new-index-image:8"],
            "manifests": [
                "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6"
            ],
        },
        "operation_results": MSG_SIGNER_OPERATION_RESULT,
        "signing_key": "sig-key",
    }
    with requests_mock.Mocker() as m:
        m.get(
            "https://quay.io/v2/src/repo/manifests/1",
            json=v2s1_manifest,
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.v2+json"},
        )
        m.get(
            "https://quay.io/v2/some-namespace/target----repo/tags/list",
            json={"name": "target-repo", "tags": ["latest-test-tag"]},
        )
        m.get(
            "https://quay.io/v2/some-namespace/external----repo/tags/list",
            json={"name": "target-repo", "tags": ["latest-test-tag"]},
        )
        m.get(
            "https://quay.io/v2/some-namespace/test_repo/tags/list",
            json={"name": "target-repo", "tags": ["latest-test-tag"]},
        )
        m.get(
            "https://quay.io/v2/some-namespace/test_repo/manifests/1.0",
            text=json.dumps(v2s1_manifest, sort_keys=True),
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.v1+json"},
        )
        m.get(
            "https://quay.io/v2/some-namespace/test_repo/manifests/latest-test-tag",
            text=json.dumps(v2s1_manifest, sort_keys=True),
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.v1+json"},
        )
        m.get(
            "https://quay.io/v2/some-namespace/external----repo/manifests/1.0",
            text=json.dumps(v2s1_manifest, sort_keys=True),
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.v1+json"},
        )
        m.get(
            "https://quay.io/v2/some-namespace/target----repo/manifests/latest-test-tag",
            text=json.dumps(v2s1_manifest, sort_keys=True),
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.v1+json"},
        )
        m.get(
            "https://quay.io/v2/some-namespace/external----repo/manifests/latest-test-tag",
            text=json.dumps(v2s1_manifest, sort_keys=True),
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.v1+json"},
        )
        m.get(
            "https://quay.io/v2/namespace/iib/manifests/v4.5-1",
            json=dest_manifest_list,
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.list.v2+json"},
        )
        m.get(
            "https://quay.io/v2/some-namespace/external----repo/manifests/latest-test-tag",
            text=json.dumps(v2s1_manifest, sort_keys=True),
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.v1+json"},
        )
        # Call for untag old signatures
        m.get(
            "https://quay.io/v2/some-namespace/somerepo/manifests/somedigest",
            text=json.dumps(v2s1_manifest, sort_keys=True),
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.v1+json"},
        )

        push_docker_instance = push_docker.PushDocker(
            [container_multiarch_push_item, operator_push_item_ok],
            hub,
            "1",
            "some-target",
            target_settings,
        )
        push_docker_instance.run()

    mock_get_docker_push_items.assert_called_once_with()
    mock_get_docker_push_items.assert_called_once_with()
    mock_check_repos_validity.assert_called_once_with(
        [container_multiarch_push_item, container_push_item_external_repos],
        hub,
        target_settings,
    )
    mock_generate_backup_mapping.assert_has_calls(
        [
            mock.call(
                [container_multiarch_push_item, container_push_item_external_repos], all_arches=True
            ),
            mock.call(
                [container_multiarch_push_item, container_push_item_external_repos], all_arches=True
            ),
        ]
    )
    mock_container_image_pusher.assert_called_once_with(
        [container_multiarch_push_item, container_push_item_external_repos],
        target_settings,
    )
    mock_push_container_images.assert_called_once_with()
    mock_security_manifest_pusher.assert_called_once_with(
        [container_multiarch_push_item, container_push_item_external_repos], target_settings
    )
    mock_security_manifest_pusher.return_value.push_security_manifests.assert_called_once_with()
    mock_operator_pusher.assert_called_once_with([operator_push_item_ok], "1", target_settings)
    mock_build_index_images.assert_called_once_with()
    mock_push_index_images.assert_called_once_with(
        {"v4.5": {"iib_result": iib_res, "signing_keys": [], "destination_tags": ["v4.5"]}},
        "timestamp",
    )
    mock_rollback.assert_not_called()


@mock.patch("pubtools._quay.push_docker.SecurityManifestPusher")
@mock.patch("pubtools._quay.push_docker.PushDocker.rollback")
@mock.patch("pubtools._quay.push_docker.OperatorPusher")
@mock.patch("pubtools._quay.push_docker.ContainerImagePusher")
@mock.patch("pubtools._quay.push_docker.PushDocker.generate_backup_mapping")
@mock.patch("pubtools._quay.push_docker.PushDocker.check_repos_validity")
@mock.patch("pubtools._quay.push_docker.PushDocker.get_operator_push_items")
@mock.patch("pubtools._quay.push_docker.PushDocker.get_docker_push_items")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_push_docker_no_operator_push_items(
    mock_quay_api_client,
    mock_get_docker_push_items,
    mock_get_operator_push_items,
    mock_check_repos_validity,
    mock_generate_backup_mapping,
    mock_container_image_pusher,
    mock_operator_pusher,
    mock_rollback,
    mock_security_manifest_pusher,
    target_settings,
    container_multiarch_push_item,
    fake_cert_key_paths,
    v2s1_manifest,
    dest_manifest_list,
    signer_wrapper_entry_point,
    signer_wrapper_run_entry_point,
):
    hub = mock.MagicMock()
    mock_push_container_images = mock.MagicMock()
    mock_container_image_pusher.return_value.push_container_images = mock_push_container_images

    mock_sign_operator_images = mock.MagicMock(return_value=[])

    iib_result = mock.MagicMock(internal_index_image_copy_resolved="registry/ns/iib@digest")
    mock_push_index_images = mock.MagicMock()
    mock_operator_pusher.return_value.push_index_images = mock_push_index_images
    mock_build_index_images = mock.MagicMock()
    mock_operator_pusher.return_value.build_index_images = mock_build_index_images

    mock_get_docker_push_items.return_value = [container_multiarch_push_item]
    mock_get_operator_push_items.return_value = []
    mock_generate_backup_mapping.side_effect = [
        (
            {
                push_docker.PushDocker.ImageData(
                    "some-ns/orig-ns----somerepo", "sometag", None, None, None
                ): ({"digest": "some-digest"}, "amd64")
            },
            ["item1", "item2"],
        ),
        (
            {
                push_docker.PushDocker.ImageData(
                    "some-ns/orig-ns----somerepo", "sometag", None, None, None
                ): ({"digest": "some-digest-new"}, "amd64")
            },
            ["item1", "item2"],
        ),
    ]
    signer_wrapper_entry_point.return_value = {
        "signer_result": {
            "status": "ok",
        },
        "operation": {
            "references": ["some-registry.com/iib-namespace/new-index-image:8"],
            "manifests": [
                "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6"
            ],
        },
        "operation_results": MSG_SIGNER_OPERATION_RESULT,
        "signing_key": "sig-key",
    }

    with requests_mock.Mocker() as m:
        m.get(
            "https://quay.io/v2/src/repo/manifests/1",
            json=v2s1_manifest,
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.v2+json"},
        )
        m.get(
            "https://quay.io/v2/some-namespace/target----repo/tags/list",
            json={"name": "target-repo", "tags": ["latest-test-tag"]},
        )
        m.get(
            "https://quay.io/v2/some-namespace/target----repo/manifests/latest-test-tag",
            text=json.dumps(v2s1_manifest, sort_keys=True),
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.v1+json"},
            request_headers={"Accept": "application/vnd.docker.distribution.manifest.v1+json"},
        )
        m.get(
            "https://quay.io/v2/namespace/iib/manifests/v4.5-1",
            json=dest_manifest_list,
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.list.v2+json"},
        )

        push_docker_instance = push_docker.PushDocker(
            [container_multiarch_push_item], hub, "1", "some-target", target_settings
        )
        push_docker_instance.run()

    mock_get_docker_push_items.assert_called_once_with()
    mock_get_docker_push_items.assert_called_once_with()
    mock_check_repos_validity.assert_called_once_with(
        [container_multiarch_push_item], hub, target_settings
    )
    mock_generate_backup_mapping.assert_has_calls(
        [
            mock.call([container_multiarch_push_item], all_arches=True),
            mock.call([container_multiarch_push_item], all_arches=True),
        ]
    )

    mock_container_image_pusher.assert_called_once_with(
        [container_multiarch_push_item], target_settings
    )
    mock_push_container_images.assert_called_once_with()

    mock_security_manifest_pusher.assert_called_once_with(
        [container_multiarch_push_item], target_settings
    )
    mock_security_manifest_pusher.return_value.push_security_manifests.assert_called_once_with()
    mock_operator_pusher.assert_not_called()
    mock_build_index_images.assert_not_called()
    mock_push_index_images.assert_not_called()
    mock_sign_operator_images.assert_not_called()
    mock_rollback.assert_not_called()


@mock.patch("pubtools._quay.push_docker.SecurityManifestPusher")
@mock.patch("pubtools._quay.push_docker.PushDocker.rollback")
@mock.patch("pubtools._quay.push_docker.OperatorPusher")
@mock.patch("pubtools._quay.push_docker.ContainerImagePusher")
@mock.patch("pubtools._quay.push_docker.PushDocker.generate_backup_mapping")
@mock.patch("pubtools._quay.push_docker.PushDocker.check_repos_validity")
@mock.patch("pubtools._quay.push_docker.PushDocker.get_operator_push_items")
@mock.patch("pubtools._quay.push_docker.PushDocker.get_docker_push_items")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_push_docker_failure_no_rollback(
    mock_quay_api_client,
    mock_get_docker_push_items,
    mock_get_operator_push_items,
    mock_check_repos_validity,
    mock_generate_backup_mapping,
    mock_container_image_pusher,
    mock_operator_pusher,
    # mock_operator_signature_handler,
    mock_rollback,
    mock_security_manifest_pusher,
    target_settings,
    fake_cert_key_paths,
    container_multiarch_push_item,
    operator_push_item_ok,
    v2s1_manifest,
    dest_manifest_list,
    signer_wrapper_entry_point,
    signer_wrapper_run_entry_point,
):
    """Rollback shouldn't be triggered as one of the index image build is succesfull."""
    hub = mock.MagicMock()
    mock_push_container_images = mock.MagicMock()
    mock_container_image_pusher.return_value.push_container_images = mock_push_container_images
    mock_sign_container_images = mock.MagicMock(return_value=([], []))

    iib_res = IIBRes(
        "registry.com/namespace/index-image@sha256:v4.5",
        "registry.com/namespace/iib@sha256:a1a1a1",
        ["v4.5-1"],
    )
    mock_build_index_images = mock.MagicMock()
    mock_build_index_images.return_value = {
        "v4.5": {"iib_result": iib_res, "signing_keys": [], "destination_tags": ["v4.5"]},
        "v4.6": {"iib_result": False, "signing_keys": [], "destination_tags": ["v4.6"]},
    }
    mock_operator_pusher.return_value.build_index_images = mock_build_index_images
    mock_push_index_images = mock.MagicMock()
    mock_operator_pusher.return_value.push_index_images = mock_push_index_images
    mock_sign_operator_images = mock.MagicMock(return_value=([], []))

    mock_get_docker_push_items.return_value = [container_multiarch_push_item]
    mock_get_operator_push_items.return_value = [operator_push_item_ok]
    mock_generate_backup_mapping.side_effect = [
        (
            {
                push_docker.PushDocker.ImageData(
                    "some-ns/orig-ns----somerepo", "sometag", None, None, None
                ): ({"digest": "some-digest"}, "amd64")
            },
            ["item1", "item2"],
        ),
        (
            {
                push_docker.PushDocker.ImageData(
                    "some-ns/orig-ns----somerepo", "sometag", None, None, None
                ): ({"digest": "some-digest-new"}, "amd64")
            },
            ["item1", "item2"],
        ),
    ]
    signer_wrapper_entry_point.return_value = {
        "signer_result": {
            "status": "ok",
        },
        "operation": {
            "references": ["some-registry.com/iib-namespace/new-index-image:8"],
            "manifests": [
                "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6"
            ],
        },
        "operation_results": MSG_SIGNER_OPERATION_RESULT,
        "signing_key": "sig-key",
    }
    with requests_mock.Mocker() as m:
        m.get(
            "https://quay.io/v2/src/repo/manifests/1",
            json=v2s1_manifest,
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.v2+json"},
        )
        m.get(
            "https://quay.io/v2/some-namespace/target----repo/tags/list",
            json={"name": "target-repo", "tags": ["latest-test-tag"]},
        )
        m.get(
            "https://quay.io/v2/some-namespace/target----repo/tags/list",
            json={"name": "target-repo", "tags": ["latest-test-tag"]},
        )
        m.get(
            "https://quay.io/v2/some-namespace/target----repo/manifests/latest-test-tag",
            text=json.dumps(v2s1_manifest, sort_keys=True),
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.v1+json"},
            request_headers={"Accept": "application/vnd.docker.distribution.manifest.v1+json"},
        )
        m.get(
            "https://quay.io/v2/namespace/iib/manifests/v4.5-1",
            json=dest_manifest_list,
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.list.v2+json"},
        )

        push_docker_instance = push_docker.PushDocker(
            [container_multiarch_push_item, operator_push_item_ok],
            hub,
            "1",
            "some-target",
            target_settings,
        )
        with pytest.raises(SystemExit):
            push_docker_instance.run()

    mock_get_docker_push_items.assert_called_once_with()
    mock_get_docker_push_items.assert_called_once_with()
    mock_check_repos_validity.assert_called_once_with(
        [container_multiarch_push_item], hub, target_settings
    )
    mock_generate_backup_mapping.assert_has_calls(
        [
            mock.call([container_multiarch_push_item], all_arches=True),
            mock.call([container_multiarch_push_item], all_arches=True),
        ]
    )

    mock_container_image_pusher.assert_called_once_with(
        [container_multiarch_push_item], target_settings
    )
    mock_push_container_images.assert_called_once_with()

    mock_security_manifest_pusher.assert_called_once_with(
        [container_multiarch_push_item], target_settings
    )
    mock_security_manifest_pusher.return_value.push_security_manifests.assert_called_once_with()
    mock_operator_pusher.assert_called_once()
    mock_build_index_images.assert_called_once()
    mock_push_index_images.assert_called_once()
    mock_rollback.assert_not_called()


@mock.patch("pubtools._quay.push_docker.SecurityManifestPusher")
@mock.patch("pubtools._quay.push_docker.PushDocker.rollback")
@mock.patch("pubtools._quay.push_docker.OperatorPusher")
@mock.patch("pubtools._quay.push_docker.ContainerImagePusher")
@mock.patch("pubtools._quay.push_docker.PushDocker.generate_backup_mapping")
@mock.patch("pubtools._quay.push_docker.PushDocker.check_repos_validity")
@mock.patch("pubtools._quay.push_docker.PushDocker.get_operator_push_items")
@mock.patch("pubtools._quay.push_docker.PushDocker.get_docker_push_items")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_push_docker_failure_rollback(
    mock_quay_api_client,
    mock_get_docker_push_items,
    mock_get_operator_push_items,
    mock_check_repos_validity,
    mock_generate_backup_mapping,
    mock_container_image_pusher,
    mock_operator_pusher,
    mock_rollback,
    mock_security_manifest_pusher,
    target_settings,
    container_multiarch_push_item,
    operator_push_item_ok,
    v2s1_manifest,
    dest_manifest_list,
    fake_cert_key_paths,
    signer_wrapper_entry_point,
    signer_wrapper_run_entry_point,
):
    hub = mock.MagicMock()
    mock_push_container_images = mock.MagicMock()
    mock_container_image_pusher.return_value.push_container_images = mock_push_container_images
    mock_build_index_images = mock.MagicMock()
    iib_result = mock.MagicMock(internal_index_image_copy_resolved="registry/ns/iib@digest")
    mock_build_index_images.return_value = {
        "v4.5": {"iib_result": False, "signing_keys": []},
        "v4.6": {"iib_result": False, "signing_keys": []},
    }
    mock_operator_pusher.return_value.build_index_images = mock_build_index_images
    mock_push_index_images = mock.MagicMock()
    mock_operator_pusher.return_value.push_index_images = mock_push_index_images

    mock_get_docker_push_items.return_value = [container_multiarch_push_item]
    mock_get_operator_push_items.return_value = [operator_push_item_ok]
    mock_generate_backup_mapping.return_value = (
        {
            push_docker.PushDocker.ImageData(
                "some-ns/orig-ns----somerepo", "sometag", None, None, None
            ): ({"digest": "some-digest"}, "amd64")
        },
        ["item1", "item2"],
    )

    push_docker_instance = push_docker.PushDocker(
        [container_multiarch_push_item, operator_push_item_ok],
        hub,
        "1",
        "some-target",
        target_settings,
    )
    signer_wrapper_entry_point.return_value = {
        "signer_result": {
            "status": "ok",
        },
        "operation": {
            "references": ["some-registry.com/iib-namespace/new-index-image:8"],
            "manifests": [
                "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6"
            ],
        },
        "operation_results": MSG_SIGNER_OPERATION_RESULT,
        "signing_key": "sig-key",
    }
    with requests_mock.Mocker() as m:
        m.get(
            "https://quay.io/v2/src/repo/manifests/1",
            json=v2s1_manifest,
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.v2+json"},
        )
        m.get(
            "https://quay.io/v2/some-namespace/target----repo/tags/list",
            json={"name": "target-repo", "tags": ["latest-test-tag"]},
        )
        m.get(
            "https://quay.io/v2/some-namespace/target----repo/tags/list",
            json={"name": "target-repo", "tags": ["latest-test-tag"]},
        )
        m.get(
            "https://quay.io/v2/some-namespace/target----repo/manifests/latest-test-tag",
            text=json.dumps(v2s1_manifest, sort_keys=True),
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.v1+json"},
            request_headers={"Accept": "application/vnd.docker.distribution.manifest.v1+json"},
        )
        m.get(
            "https://quay.io/v2/namespace/iib/manifests/v4.5-1",
            json=dest_manifest_list,
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.list.v2+json"},
        )
        with pytest.raises(SystemExit):
            push_docker_instance.run()

    mock_get_docker_push_items.assert_called_once_with()
    mock_get_docker_push_items.assert_called_once_with()
    mock_check_repos_validity.assert_called_once_with(
        [container_multiarch_push_item], hub, target_settings
    )
    mock_generate_backup_mapping.assert_called_once_with(
        [container_multiarch_push_item], all_arches=True
    )
    mock_container_image_pusher.assert_called_once_with(
        [container_multiarch_push_item], target_settings
    )
    mock_push_container_images.assert_called_once_with()
    mock_security_manifest_pusher.assert_called_once_with(
        [container_multiarch_push_item], target_settings
    )
    mock_security_manifest_pusher.return_value.push_security_manifests.assert_called_once_with()
    mock_operator_pusher.assert_called_once()
    mock_build_index_images.assert_called_once()
    mock_push_index_images.assert_called_once()
    mock_rollback.assert_called_once_with(
        {
            push_docker.PushDocker.ImageData(
                "some-ns/orig-ns----somerepo", "sometag", None, None, None
            ): {"digest": "some-digest"}
        },
        ["item1", "item2"],
    )


@mock.patch("pubtools._quay.push_docker.SecurityManifestPusher")
@mock.patch("pubtools._quay.push_docker.PushDocker.rollback")
@mock.patch("pubtools._quay.push_docker.OperatorPusher")
@mock.patch("pubtools._quay.push_docker.ContainerImagePusher")
@mock.patch("pubtools._quay.push_docker.PushDocker.generate_backup_mapping")
@mock.patch("pubtools._quay.push_docker.PushDocker.check_repos_validity")
@mock.patch("pubtools._quay.push_docker.PushDocker.get_operator_push_items")
@mock.patch("pubtools._quay.push_docker.PushDocker.get_docker_push_items")
@mock.patch("pubtools._quay.push_docker.QuayApiClient")
def test_push_docker_failure_fbc_rollback(
    mock_quay_api_client,
    mock_get_docker_push_items,
    mock_get_operator_push_items,
    mock_check_repos_validity,
    mock_generate_backup_mapping,
    mock_container_image_pusher,
    mock_operator_pusher,
    mock_rollback,
    mock_security_manifest_pusher,
    target_settings,
    container_multiarch_push_item,
    operator_push_item_fbc_inconsistent,
    v2s1_manifest,
    dest_manifest_list,
    fake_cert_key_paths,
    signer_wrapper_entry_point,
    signer_wrapper_run_entry_point,
):
    operator_push_item_fbc_inconsistent.errors = {"repo": "error"}
    hub = mock.MagicMock()
    mock_push_container_images = mock.MagicMock()
    mock_container_image_pusher.return_value.push_container_images = mock_push_container_images
    mock_sign_container_images = mock.MagicMock(return_value=([], []))
    mock_build_index_images = mock.MagicMock()
    iib_result = mock.MagicMock(internal_index_image_copy_resolved="registry/ns/iib@digest")
    mock_build_index_images.return_value = {
        "v4.5": {"iib_result": False, "signing_keys": []},
        "v4.6": {"iib_result": False, "signing_keys": []},
    }
    mock_operator_pusher.return_value.build_index_images = mock_build_index_images
    mock_push_index_images = mock.MagicMock()
    mock_operator_pusher.return_value.push_index_images = mock_push_index_images

    mock_get_docker_push_items.return_value = [container_multiarch_push_item]
    mock_get_operator_push_items.return_value = [operator_push_item_fbc_inconsistent]
    mock_generate_backup_mapping.return_value = (
        {
            push_docker.PushDocker.ImageData(
                "some-ns/orig-ns----somerepo", "sometag", None, None, None
            ): ({"digest": "some-digest"}, "amd64")
        },
        ["item1", "item2"],
    )

    push_docker_instance = push_docker.PushDocker(
        [container_multiarch_push_item, operator_push_item_fbc_inconsistent],
        hub,
        "1",
        "some-target",
        target_settings,
    )
    signer_wrapper_entry_point.return_value = {
        "signer_result": {
            "status": "ok",
        },
        "operation": {
            "references": ["some-registry.com/iib-namespace/new-index-image:8"],
            "manifests": [
                "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6"
            ],
        },
        "operation_results": MSG_SIGNER_OPERATION_RESULT,
        "signing_key": "sig-key",
    }
    with requests_mock.Mocker() as m:
        m.get(
            "https://quay.io/v2/src/repo/manifests/1",
            json=v2s1_manifest,
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.v2+json"},
        )
        m.get(
            "https://quay.io/v2/some-namespace/target----repo/tags/list",
            json={"name": "target-repo", "tags": ["latest-test-tag"]},
        )
        m.get(
            "https://quay.io/v2/some-namespace/target----repo/tags/list",
            json={"name": "target-repo", "tags": ["latest-test-tag"]},
        )
        m.get(
            "https://quay.io/v2/some-namespace/target----repo/manifests/latest-test-tag",
            text=json.dumps(v2s1_manifest, sort_keys=True),
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.v1+json"},
            request_headers={"Accept": "application/vnd.docker.distribution.manifest.v1+json"},
        )
        m.get(
            "https://quay.io/v2/namespace/iib/manifests/v4.5-1",
            json=dest_manifest_list,
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.list.v2+json"},
        )
        with pytest.raises(SystemExit):
            push_docker_instance.run()

    mock_get_docker_push_items.assert_called_once_with()
    mock_get_docker_push_items.assert_called_once_with()
    mock_check_repos_validity.assert_called_once_with(
        [container_multiarch_push_item], hub, target_settings
    )
    mock_generate_backup_mapping.assert_called_once_with(
        [container_multiarch_push_item], all_arches=True
    )
    mock_container_image_pusher.assert_called_once_with(
        [container_multiarch_push_item], target_settings
    )
    mock_push_container_images.assert_called_once_with()
    mock_security_manifest_pusher.assert_called_once_with(
        [container_multiarch_push_item], target_settings
    )
    mock_security_manifest_pusher.return_value.push_security_manifests.assert_called_once_with()
    mock_operator_pusher.assert_called_once()
    mock_build_index_images.assert_called_once()
    mock_push_index_images.assert_not_called()
    mock_rollback.assert_called_once_with(
        {
            push_docker.PushDocker.ImageData(
                "some-ns/orig-ns----somerepo", "sometag", None, None, None
            ): {"digest": "some-digest"}
        },
        ["item1", "item2"],
    )


@mock.patch("pubtools._quay.push_docker.PushDocker")
def test_mod_entrypoint(
    mock_push_docker,
    container_multiarch_push_item,
    operator_push_item_ok,
    target_settings,
):
    hub = mock.MagicMock()
    mock_run = mock.MagicMock()
    mock_run.return_value = ["repo1", "repo2"]
    mock_push_docker.return_value.run = mock_run

    push_docker.mod_entry_point(
        [container_multiarch_push_item, operator_push_item_ok],
        hub,
        "1",
        "some-target",
        target_settings,
    )
    mock_push_docker.assert_called_once_with(
        [container_multiarch_push_item, operator_push_item_ok],
        hub,
        "1",
        "some-target",
        target_settings,
    )
    mock_run.assert_called_once_with()



import mock
import paramiko
import pytest
import logging

from pubtools._quay import command_executor
from .utils.misc import compare_logs


def test_local_executor_init():
    executor = command_executor.LocalExecutor({"some_param": "value"})

    assert executor.params["some_param"] == "value"
    assert executor.params["universal_newlines"] is True
    assert executor.params["stderr"] == -1
    assert executor.params["stdout"] == -1
    assert executor.params["stdin"] == -1


@mock.patch("pubtools._quay.command_executor.subprocess.Popen")
def test_local_executor_run(mock_popen):
    executor = command_executor.LocalExecutor({"some_param": "value"})

    mock_communicate = mock.MagicMock()
    mock_communicate.return_value = ("outlog", "errlog")
    mock_popen.return_value.communicate = mock_communicate
    mock_popen.return_value.returncode = 0

    out, err = executor._run_cmd("pwd", stdin="input")
    assert out == "outlog"
    assert err == "errlog"
    mock_popen.assert_called_once_with(
        ["pwd"],
        some_param="value",
        universal_newlines=True,
        stderr=-1,
        stdout=-1,
        stdin=-1,
    )
    mock_communicate.assert_called_once_with(input="input")


@mock.patch("pubtools._quay.command_executor.subprocess.Popen")
def test_local_executor_context_manager(mock_popen):
    mock_communicate = mock.MagicMock()
    mock_communicate.return_value = ("outlog", "errlog")
    mock_popen.return_value.communicate = mock_communicate
    mock_popen.return_value.returncode = 0

    with command_executor.LocalExecutor({"some_param": "value"}) as executor:
        out, err = executor._run_cmd("pwd", stdin="input")
    assert out == "outlog"
    assert err == "errlog"
    mock_popen.assert_called_once_with(
        ["pwd"],
        some_param="value",
        universal_newlines=True,
        stderr=-1,
        stdout=-1,
        stdin=-1,
    )
    mock_communicate.assert_called_once_with(input="input")


@mock.patch("pubtools._quay.command_executor.subprocess.Popen")
def test_local_executor_run_error(mock_popen):
    executor = command_executor.LocalExecutor({"some_param": "value"})

    mock_communicate = mock.MagicMock()
    mock_communicate.return_value = ("outlog", "errlog")
    mock_popen.return_value.communicate = mock_communicate
    mock_popen.return_value.returncode = -1

    with pytest.raises(RuntimeError, match="An error has occured when executing.*"):
        executor._run_cmd("pwd", stdin="input")


@mock.patch("pubtools._quay.command_executor.subprocess.Popen")
def test_local_executor_run_long_error(mock_popen, caplog):
    caplog.set_level(logging.ERROR)
    executor = command_executor.LocalExecutor({"some_param": "value"})

    err_msg = " ".join(["Very long error message."] * 40)

    mock_communicate = mock.MagicMock()
    mock_communicate.return_value = ("outlog", err_msg)
    mock_popen.return_value.communicate = mock_communicate
    mock_popen.return_value.returncode = -1

    expected_logs = [
        ".*failed with the following error:",
        "    Very long error message. Very long error message. Very long error message. "
        "Very long error message. Very long error message. Very long error message. "
        "Very long error message. Very long error message.",
        "    Very long error message. Very long error message. Very long error message. "
        "Very long error message. Very long error message. Very long error message. "
        "Very long error message. Very long error message.",
        "    Very long error message. Very long error message. Very long error message. "
        "Very long error message. Very long error message. Very long error message. "
        "Very long error message. Very long error message.",
        "    Very long error message. Very long error message. Very long error message. "
        "Very long error message. Very long error message. Very long error message. "
        "Very long error message. Very long error message.",
        "    Very long error message. Very long error message. Very long error message. "
        "Very long error message. Very long error message. Very long error message. "
        "Very long error message. Very long error message.",
    ]

    with pytest.raises(RuntimeError, match="An error has occured when executing.*"):
        executor._run_cmd("pwd", stdin="input")

    compare_logs(caplog, expected_logs)


@mock.patch("pubtools._quay.command_executor.subprocess.Popen")
def test_local_executor_run_error_custom_message(mock_popen):
    executor = command_executor.LocalExecutor({"some_param": "value"})

    mock_communicate = mock.MagicMock()
    mock_communicate.return_value = ("outlog", "errlog")
    mock_popen.return_value.communicate = mock_communicate
    mock_popen.return_value.returncode = -1

    with pytest.raises(RuntimeError, match="Custom error"):
        executor._run_cmd("pwd", stdin="input", err_msg="Custom error")


@mock.patch("pubtools._quay.command_executor.subprocess.Popen")
def test_local_executor_run_tolerate_err(mock_popen):
    executor = command_executor.LocalExecutor({"some_param": "value"})

    mock_communicate = mock.MagicMock()
    mock_communicate.return_value = ("outlog", "errlog")
    mock_popen.return_value.communicate = mock_communicate
    mock_popen.return_value.returncode = -1

    out, err = executor._run_cmd("pwd", stdin="input", tolerate_err=True)
    assert out == "outlog"
    assert err == "errlog"
    mock_popen.assert_called_once_with(
        ["pwd"],
        some_param="value",
        universal_newlines=True,
        stderr=-1,
        stdout=-1,
        stdin=-1,
    )
    mock_communicate.assert_called_once_with(input="input")


def test_remote_executor_init():
    executor = command_executor.RemoteExecutor(
        "127.0.0.1",
        username="dummy",
        key_filename="path/to/file.key",
        password="123456",
        accept_unknown_host=False,
    )

    assert executor.hostname == "127.0.0.1"
    assert executor.username == "dummy"
    assert executor.key_filename == "path/to/file.key"
    assert executor.password == "123456"
    assert isinstance(executor.missing_host_policy, paramiko.client.RejectPolicy)

    executor2 = command_executor.RemoteExecutor("127.0.0.1")

    assert isinstance(executor2.missing_host_policy, paramiko.client.WarningPolicy)


@mock.patch("pubtools._quay.command_executor.paramiko.client.SSHClient")
def test_remote_executor_run(mock_sshclient):
    executor = command_executor.RemoteExecutor(
        "127.0.0.1",
        username="dummy",
        key_filename="path/to/file.key",
        password="123456",
        accept_unknown_host=True,
    )

    mock_load_host_keys = mock.MagicMock()
    mock_sshclient.return_value.load_system_host_keys = mock_load_host_keys
    mock_set_keys = mock.MagicMock()
    mock_sshclient.return_value.set_missing_host_key_policy = mock_set_keys
    mock_connect = mock.MagicMock()
    mock_sshclient.return_value.connect = mock_connect

    mock_in = mock.MagicMock()
    mock_out = mock.MagicMock()
    mock_out.read.return_value = b"outlog"
    mock_recv_exit_status = mock.MagicMock()
    mock_recv_exit_status.return_value = 0
    mock_out.channel.recv_exit_status = mock_recv_exit_status
    mock_err = mock.MagicMock()
    mock_err.read.return_value = b"errlog"
    mock_send = mock.MagicMock()
    mock_shutdown_write = mock.MagicMock()
    mock_in.channel.send = mock_send
    mock_in.channel.shutdown_write = mock_shutdown_write
    mock_exec_command = mock.MagicMock()
    mock_exec_command.return_value = (mock_in, mock_out, mock_err)
    mock_sshclient.return_value.exec_command = mock_exec_command

    out, err = executor._run_cmd("skopeo", stdin="input")

    mock_load_host_keys.assert_called_once()
    assert mock_set_keys.call_count == 1
    assert isinstance(mock_set_keys.call_args[0][0], paramiko.client.WarningPolicy)
    # mock_set_keys.assert_called_once_with(paramiko.client.WarningPolicy)
    mock_connect.assert_called_once_with(
        "127.0.0.1",
        username="dummy",
        password="123456",
        port=22,
        key_filename="path/to/file.key",
    )
    mock_exec_command.assert_called_once_with("'skopeo --authfile $HOME/.docker/config.json'")
    mock_send.assert_called_once_with("input")
    mock_shutdown_write.assert_called_once()
    mock_recv_exit_status.assert_called_once()

    assert out == "outlog"
    assert err == "errlog"


@mock.patch("pubtools._quay.command_executor.paramiko.client.SSHClient")
def test_remote_executor_run_error(mock_sshclient):
    executor = command_executor.RemoteExecutor(
        "127.0.0.1",
        username="dummy",
        key_filename="path/to/file.key",
        password="123456",
        accept_unknown_host=True,
    )

    mock_load_host_keys = mock.MagicMock()
    mock_sshclient.return_value.load_system_host_keys = mock_load_host_keys
    mock_set_keys = mock.MagicMock()
    mock_sshclient.return_value.set_missing_host_key_policy = mock_set_keys
    mock_connect = mock.MagicMock()
    mock_sshclient.return_value.connect = mock_connect

    mock_in = mock.MagicMock()
    mock_out = mock.MagicMock()
    mock_out.read.return_value = b"outlog"
    mock_recv_exit_status = mock.MagicMock()
    mock_recv_exit_status.return_value = -1
    mock_out.channel.recv_exit_status = mock_recv_exit_status
    mock_err = mock.MagicMock()
    mock_err.read.return_value = b"errlog"
    mock_send = mock.MagicMock()
    mock_shutdown_write = mock.MagicMock()
    mock_in.channel.send = mock_send
    mock_in.channel.shutdown_write = mock_shutdown_write
    mock_exec_command = mock.MagicMock()
    mock_exec_command.return_value = (mock_in, mock_out, mock_err)
    mock_sshclient.return_value.exec_command = mock_exec_command

    with pytest.raises(RuntimeError, match="An error has occured when executing.*"):
        executor._run_cmd("pwd", stdin="input")


@mock.patch("pubtools._quay.command_executor.os.path.isdir")
@mock.patch("pubtools._quay.command_executor.docker.tls.TLSConfig")
@mock.patch("pubtools._quay.command_executor.docker.APIClient")
def test_container_executor_init(mock_api_client, mock_tls_config, mock_isdir):
    mock_create_container = mock.MagicMock()
    mock_create_container.return_value = {"Id": "123"}
    mock_api_client.return_value.create_container = mock_create_container
    mock_start = mock.MagicMock()
    mock_api_client.return_value.start = mock_start
    mock_remove_container = mock.MagicMock()
    mock_api_client.return_value.remove_container = mock_remove_container
    mock_isdir.return_value = True

    with command_executor.ContainerExecutor(
        "quay.io/some/image:1",
        base_url="some-url.com",
        timeout=120,
        verify_tls=True,
        cert_path="/some/path",
        registry_username="registry_user",
        registry_password="registry_passwd",
    ):
        pass

    mock_tls_config.assert_called_once_with(
        client_cert=("/some/path/cert.pem", "/some/path/key.pem"), verify="/some/path/ca.pem"
    )
    mock_api_client.assert_called_once_with(
        base_url="some-url.com", version="auto", timeout=120, tls=mock_tls_config.return_value
    )
    mock_create_container.assert_called_once_with("quay.io/some/image:1", detach=True, tty=True)
    mock_start.assert_called_once_with("123")
    mock_remove_container.assert_called_once_with("123", force=True)
    mock_api_client.return_value.login.assert_called_once_with(
        username="registry_user", password="registry_passwd", registry="quay.io", reauth=True
    )
    mock_api_client.return_value.pull.assert_called_once_with("quay.io/some/image", tag="1")


@mock.patch("pubtools._quay.command_executor.os.path.isdir")
@mock.patch("pubtools._quay.command_executor.docker.tls.TLSConfig")
@mock.patch("pubtools._quay.command_executor.docker.APIClient")
def test_container_executor_run_cmd(mock_api_client, mock_tls_config, mock_isdir):
    mock_create_container = mock.MagicMock()
    mock_create_container.return_value = {"Id": "123"}
    mock_api_client.return_value.create_container = mock_create_container
    mock_start = mock.MagicMock()
    mock_api_client.return_value.start = mock_start
    mock_remove_container = mock.MagicMock()
    mock_api_client.return_value.remove_container = mock_remove_container
    mock_isdir.return_value = True

    mock_exec_create = mock.MagicMock()
    mock_exec_create.return_value = {"Id": "321"}
    mock_api_client.return_value.exec_create = mock_exec_create
    mock_exec_start = mock.MagicMock()
    mock_exec_start.return_value = b"some output"
    mock_api_client.return_value.exec_start = mock_exec_start
    mock_exec_inspect = mock.MagicMock()
    mock_exec_inspect.return_value = {"ExitCode": 0}
    mock_api_client.return_value.exec_inspect = mock_exec_inspect

    with command_executor.ContainerExecutor(
        "quay.io/some/image:1",
        base_url="some-url.com",
        timeout=120,
        verify_tls=True,
        cert_path="/some/path",
    ) as executor:
        stdout, stderr = executor._run_cmd("cat file.txt")

    assert stdout == "some output"
    assert stderr == "some output"

    mock_exec_create.assert_called_once_with("123", "cat file.txt")
    mock_exec_start.assert_called_once_with("321")
    mock_exec_inspect.assert_called_once_with("321")


@mock.patch("pubtools._quay.command_executor.os.path.isdir")
@mock.patch("pubtools._quay.command_executor.docker.tls.TLSConfig")
@mock.patch("pubtools._quay.command_executor.docker.APIClient")
def test_container_executor_run_cmd_error(mock_api_client, mock_tls_config, mock_isdir):
    mock_create_container = mock.MagicMock()
    mock_create_container.return_value = {"Id": "123"}
    mock_api_client.return_value.create_container = mock_create_container
    mock_start = mock.MagicMock()
    mock_api_client.return_value.start = mock_start
    mock_remove_container = mock.MagicMock()
    mock_api_client.return_value.remove_container = mock_remove_container
    mock_isdir.return_value = True

    mock_exec_create = mock.MagicMock()
    mock_exec_create.return_value = {"Id": "321"}
    mock_api_client.return_value.exec_create = mock_exec_create
    mock_exec_start = mock.MagicMock()
    mock_exec_start.return_value = b"some output"
    mock_api_client.return_value.exec_start = mock_exec_start
    mock_exec_inspect = mock.MagicMock()
    mock_exec_inspect.return_value = {"ExitCode": 1}
    mock_api_client.return_value.exec_inspect = mock_exec_inspect

    with command_executor.ContainerExecutor(
        "quay.io/some/image:1",
        base_url="some-url.com",
        timeout=120,
        verify_tls=True,
        cert_path="/some/path",
    ) as executor:
        with pytest.raises(RuntimeError, match="An error has occured when executing a command."):
            stdout, stderr = executor._run_cmd("cat file.txt")


@mock.patch("pubtools._quay.command_executor.os.path.isdir")
@mock.patch("pubtools._quay.command_executor.docker.tls.TLSConfig")
@mock.patch("pubtools._quay.command_executor.docker.APIClient")
def test_container_executor_run_cmd_error_tolerate_error(
    mock_api_client, mock_tls_config, mock_isdir
):
    mock_create_container = mock.MagicMock()
    mock_create_container.return_value = {"Id": "123"}
    mock_api_client.return_value.create_container = mock_create_container
    mock_start = mock.MagicMock()
    mock_api_client.return_value.start = mock_start
    mock_remove_container = mock.MagicMock()
    mock_api_client.return_value.remove_container = mock_remove_container
    mock_isdir.return_value = True

    mock_exec_create = mock.MagicMock()
    mock_exec_create.return_value = {"Id": "321"}
    mock_api_client.return_value.exec_create = mock_exec_create
    mock_exec_start = mock.MagicMock()
    mock_exec_start.return_value = b"some output"
    mock_api_client.return_value.exec_start = mock_exec_start
    mock_exec_inspect = mock.MagicMock()
    mock_exec_inspect.return_value = {"ExitCode": 1}
    mock_api_client.return_value.exec_inspect = mock_exec_inspect

    with command_executor.ContainerExecutor(
        "quay.io/some/image:1",
        base_url="some-url.com",
        timeout=120,
        verify_tls=True,
        cert_path="/some/path",
    ) as executor:
        executor._run_cmd("cat file.txt", tolerate_err=True)


@mock.patch("pubtools._quay.command_executor.os.path.isdir")
@mock.patch("pubtools._quay.command_executor.docker.tls.TLSConfig")
@mock.patch("pubtools._quay.command_executor.docker.APIClient")
def test_container_executor_run_cmd_no_output(mock_api_client, mock_tls_config, mock_isdir):
    mock_create_container = mock.MagicMock()
    mock_create_container.return_value = {"Id": "123"}
    mock_api_client.return_value.create_container = mock_create_container
    mock_start = mock.MagicMock()
    mock_api_client.return_value.start = mock_start
    mock_remove_container = mock.MagicMock()
    mock_api_client.return_value.remove_container = mock_remove_container
    mock_isdir.return_value = True

    mock_exec_create = mock.MagicMock()
    mock_exec_create.return_value = {"Id": "321"}
    mock_api_client.return_value.exec_create = mock_exec_create
    mock_exec_start = mock.MagicMock()
    mock_exec_start.return_value = None
    mock_api_client.return_value.exec_start = mock_exec_start
    mock_exec_inspect = mock.MagicMock()
    mock_exec_inspect.return_value = {"ExitCode": 0}
    mock_api_client.return_value.exec_inspect = mock_exec_inspect

    with command_executor.ContainerExecutor(
        "quay.io/some/image:1",
        base_url="some-url.com",
        timeout=120,
        verify_tls=True,
        cert_path="/some/path",
    ) as executor:
        stdout, stderr = executor._run_cmd("cat file.txt")

    assert stdout == ""
    assert stderr == ""


@mock.patch("pubtools._quay.command_executor.time.time")
@mock.patch("pubtools._quay.command_executor.tarfile.TarInfo")
@mock.patch("pubtools._quay.command_executor.tarfile.TarFile")
@mock.patch("pubtools._quay.command_executor.io.BytesIO")
@mock.patch("pubtools._quay.command_executor.os.path.isdir")
@mock.patch("pubtools._quay.command_executor.docker.tls.TLSConfig")
@mock.patch("pubtools._quay.command_executor.docker.APIClient")
def test_container_executor_add_file(
    mock_api_client,
    mock_tls_config,
    mock_isdir,
    mock_bytesio,
    mock_tarfile,
    mock_tarinfo,
    mock_time,
):
    mock_create_container = mock.MagicMock()
    mock_create_container.return_value = {"Id": "123"}
    mock_api_client.return_value.create_container = mock_create_container
    mock_start = mock.MagicMock()
    mock_api_client.return_value.start = mock_start
    mock_remove_container = mock.MagicMock()
    mock_api_client.return_value.remove_container = mock_remove_container
    mock_isdir.return_value = True

    mock_add_file = mock.MagicMock()
    mock_tarfile.return_value.addfile = mock_add_file
    mock_close = mock.MagicMock()
    mock_tarfile.return_value.close = mock_close
    mock_bytesio1 = mock.MagicMock()
    mock_bytesio2 = mock.MagicMock()
    mock_bytesio.side_effect = [mock_bytesio1, mock_bytesio2]
    mock_seek = mock.MagicMock()
    mock_bytesio1.seek = mock_seek
    mock_put_archive = mock.MagicMock()
    mock_put_archive.return_value = True
    mock_api_client.return_value.put_archive = mock_put_archive

    with command_executor.ContainerExecutor(
        "quay.io/some/image:1",
        base_url="some-url.com",
        timeout=120,
        verify_tls=True,
        cert_path="/some/path",
    ) as executor:
        executor._add_file("abcdefg", "some-file.txt")

    assert mock_bytesio.call_count == 2
    mock_tarfile.assert_called_once_with(fileobj=mock_bytesio1, mode="w")
    mock_tarinfo.assert_called_once_with(name="some-file.txt")
    mock_add_file.assert_called_once_with(mock_tarinfo.return_value, mock_bytesio2)
    mock_close.assert_called_once_with()
    mock_seek.assert_called_once_with(0)
    mock_put_archive.assert_called_once_with(
        container="123", path="/tmp", data=mock_bytesio1  # nosec B108
    )


@mock.patch("pubtools._quay.command_executor.time.time")
@mock.patch("pubtools._quay.command_executor.tarfile.TarInfo")
@mock.patch("pubtools._quay.command_executor.tarfile.TarFile")
@mock.patch("pubtools._quay.command_executor.io.BytesIO")
@mock.patch("pubtools._quay.command_executor.os.path.isdir")
@mock.patch("pubtools._quay.command_executor.docker.tls.TLSConfig")
@mock.patch("pubtools._quay.command_executor.docker.APIClient")
def test_container_executor_add_file_fail(
    mock_api_client,
    mock_tls_config,
    mock_isdir,
    mock_bytesio,
    mock_tarfile,
    mock_tarinfo,
    mock_time,
):
    mock_create_container = mock.MagicMock()
    mock_create_container.return_value = {"Id": "123"}
    mock_api_client.return_value.create_container = mock_create_container
    mock_start = mock.MagicMock()
    mock_api_client.return_value.start = mock_start
    mock_remove_container = mock.MagicMock()
    mock_api_client.return_value.remove_container = mock_remove_container
    mock_isdir.return_value = True

    mock_add_file = mock.MagicMock()
    mock_tarfile.return_value.addfile = mock_add_file
    mock_close = mock.MagicMock()
    mock_tarfile.return_value.close = mock_close
    mock_bytesio1 = mock.MagicMock()
    mock_bytesio2 = mock.MagicMock()
    mock_bytesio.side_effect = [mock_bytesio1, mock_bytesio2]
    mock_seek = mock.MagicMock()
    mock_bytesio1.seek = mock_seek
    mock_put_archive = mock.MagicMock()
    mock_put_archive.return_value = False
    mock_api_client.return_value.put_archive = mock_put_archive

    with command_executor.ContainerExecutor(
        "quay.io/some/image:1",
        base_url="some-url.com",
        timeout=120,
        verify_tls=True,
        cert_path="/some/path",
    ) as executor:
        with pytest.raises(RuntimeError, match="File was not successfully added to the container"):
            executor._add_file("abcdefg", "some-file.txt")


@mock.patch("random.SystemRandom.choice")
@mock.patch("pubtools._quay.command_executor.ContainerExecutor._add_file")
@mock.patch("pubtools._quay.command_executor.ContainerExecutor._run_cmd")
@mock.patch("pubtools._quay.command_executor.os.path.isdir")
@mock.patch("pubtools._quay.command_executor.docker.tls.TLSConfig")
@mock.patch("pubtools._quay.command_executor.docker.APIClient")
def test_container_executor_skopeo_login(
    mock_api_client, mock_tls_config, mock_isdir, mock_run_cmd, mock_add_file, mock_choice
):
    mock_create_container = mock.MagicMock()
    mock_create_container.return_value = {"Id": "123"}
    mock_api_client.return_value.create_container = mock_create_container
    mock_start = mock.MagicMock()
    mock_api_client.return_value.start = mock_start
    mock_remove_container = mock.MagicMock()
    mock_api_client.return_value.remove_container = mock_remove_container
    mock_isdir.return_value = True

    mock_run_cmd.side_effect = [("not logged in", "nothing"), ("Login Succeeded", "nothing")]
    mock_choice.return_value = "0"

    with command_executor.ContainerExecutor(
        "quay.io/some/image:1",
        base_url="some-url.com",
        timeout=120,
        verify_tls=True,
        cert_path="/some/path",
    ) as executor:
        executor.skopeo_login("some-host", "some-name", "some-password")

    assert mock_run_cmd.call_count == 2
    assert mock_run_cmd.call_args_list[0] == mock.call(
        "skopeo login --get-login some-host", tolerate_err=True
    )
    assert mock_run_cmd.call_args_list[1] == mock.call(
        " sh -c 'cat /tmp/skopeo_password-0000000000.txt | skopeo login --authfile "
        "$HOME/.docker/config.json "
        '-u "some-name" --password-stdin some-host\''
    )
    mock_add_file.assert_called_once_with("some-password", "skopeo_password-0000000000.txt")


@mock.patch("pubtools._quay.command_executor.ContainerExecutor._add_file")
@mock.patch("pubtools._quay.command_executor.ContainerExecutor._run_cmd")
@mock.patch("pubtools._quay.command_executor.os.path.isdir")
@mock.patch("pubtools._quay.command_executor.docker.tls.TLSConfig")
@mock.patch("pubtools._quay.command_executor.docker.APIClient")
def test_container_executor_skopeo_login_already_logged(
    mock_api_client, mock_tls_config, mock_isdir, mock_run_cmd, mock_add_file
):
    mock_create_container = mock.MagicMock()
    mock_create_container.return_value = {"Id": "123"}
    mock_api_client.return_value.create_container = mock_create_container
    mock_start = mock.MagicMock()
    mock_api_client.return_value.start = mock_start
    mock_remove_container = mock.MagicMock()
    mock_api_client.return_value.remove_container = mock_remove_container
    mock_isdir.return_value = True

    mock_run_cmd.return_value = ("some-name", "nothing")

    with command_executor.ContainerExecutor(
        "quay.io/some/image:1",
        base_url="some-url.com",
        timeout=120,
        verify_tls=True,
        cert_path="/some/path",
    ) as executor:
        executor.skopeo_login("some-host", "some-name", "some-password")

    mock_run_cmd.assert_called_once_with("skopeo login --get-login some-host", tolerate_err=True)
    mock_add_file.assert_not_called()


@mock.patch("pubtools._quay.command_executor.ContainerExecutor._add_file")
@mock.patch("pubtools._quay.command_executor.ContainerExecutor._run_cmd")
@mock.patch("pubtools._quay.command_executor.os.path.isdir")
@mock.patch("pubtools._quay.command_executor.docker.tls.TLSConfig")
@mock.patch("pubtools._quay.command_executor.docker.APIClient")
def test_container_executor_skopeo_login_missing_credential(
    mock_api_client, mock_tls_config, mock_isdir, mock_run_cmd, mock_add_file
):
    mock_create_container = mock.MagicMock()
    mock_create_container.return_value = {"Id": "123"}
    mock_api_client.return_value.create_container = mock_create_container
    mock_start = mock.MagicMock()
    mock_api_client.return_value.start = mock_start
    mock_remove_container = mock.MagicMock()
    mock_api_client.return_value.remove_container = mock_remove_container
    mock_isdir.return_value = True

    mock_run_cmd.side_effect = [("not logged in", "nothing"), ("Login Succeeded", "nothing")]

    with command_executor.ContainerExecutor(
        "quay.io/some/image:1",
        base_url="some-url.com",
        timeout=120,
        verify_tls=True,
        cert_path="/some/path",
    ) as executor:
        with pytest.raises(ValueError, match="Skopeo login credentials are not present.*"):
            executor.skopeo_login("some-host", "some-name")

    mock_run_cmd.assert_called_once_with("skopeo login --get-login some-host", tolerate_err=True)
    mock_add_file.assert_not_called()


@mock.patch("random.SystemRandom.choice")
@mock.patch("pubtools._quay.command_executor.ContainerExecutor._add_file")
@mock.patch("pubtools._quay.command_executor.ContainerExecutor._run_cmd")
@mock.patch("pubtools._quay.command_executor.os.path.isdir")
@mock.patch("pubtools._quay.command_executor.docker.tls.TLSConfig")
@mock.patch("pubtools._quay.command_executor.docker.APIClient")
def test_container_executor_skopeo_login_fail(
    mock_api_client, mock_tls_config, mock_isdir, mock_run_cmd, mock_add_file, mock_choice
):
    mock_create_container = mock.MagicMock()
    mock_create_container.return_value = {"Id": "123"}
    mock_api_client.return_value.create_container = mock_create_container
    mock_start = mock.MagicMock()
    mock_api_client.return_value.start = mock_start
    mock_remove_container = mock.MagicMock()
    mock_api_client.return_value.remove_container = mock_remove_container
    mock_isdir.return_value = True
    mock_choice.return_value = "0"

    mock_run_cmd.side_effect = [("not logged in", "nothing"), ("Login Failed", "nothing")]

    with command_executor.ContainerExecutor(
        "quay.io/some/image:1",
        base_url="some-url.com",
        timeout=120,
        verify_tls=True,
        cert_path="/some/path",
    ) as executor:
        with pytest.raises(RuntimeError, match="Login command didn't generate expected output.*"):
            executor.skopeo_login("some-host", "some-name", "some-password")

    assert mock_run_cmd.call_count == 2
    assert mock_run_cmd.call_args_list[0] == mock.call(
        "skopeo login --get-login some-host", tolerate_err=True
    )
    assert mock_run_cmd.call_args_list[1] == mock.call(
        " sh -c 'cat /tmp/skopeo_password-0000000000.txt | skopeo login --authfile "
        "$HOME/.docker/config.json "
        '-u "some-name" --password-stdin some-host\''
    )
    mock_add_file.assert_called_once_with("some-password", "skopeo_password-0000000000.txt")


@mock.patch("pubtools._quay.command_executor.LocalExecutor._run_cmd")
def test_skopeo_login_already_logged(mock_run_cmd):
    executor = command_executor.LocalExecutor()

    mock_run_cmd.return_value = ("quay_user", "")
    executor.skopeo_login("quay_host", "quay_user", "quay_token")
    mock_run_cmd.assert_called_once_with("skopeo login --get-login quay_host", tolerate_err=True)


@mock.patch("pubtools._quay.command_executor.LocalExecutor._run_cmd")
def test_skopeo_login_missing_credentials(mock_run_cmd):
    executor = command_executor.LocalExecutor()

    mock_run_cmd.return_value = ("not logged into quay", "")
    with pytest.raises(ValueError, match=".*login credentials are not present.*"):
        executor.skopeo_login("some-host")


@mock.patch("pubtools._quay.command_executor.LocalExecutor._run_cmd")
def test_skopeo_login_success(mock_run_cmd):
    executor = command_executor.LocalExecutor()

    mock_run_cmd.side_effect = [("not logged into quay", ""), ("Login Succeeded", "")]
    executor.skopeo_login("quay_host", "quay_user", "quay_token")
    assert mock_run_cmd.call_args_list == [
        mock.call("skopeo login --get-login quay_host", tolerate_err=True),
        mock.call(
            "skopeo login -u quay_user --password-stdin quay_host",
            stdin="quay_token",
        ),
    ]


@mock.patch("pubtools._quay.command_executor.LocalExecutor._run_cmd")
def test_skopeo_login_failed(mock_run_cmd):
    executor = command_executor.LocalExecutor()

    mock_run_cmd.side_effect = [("not logged into quay", ""), ("", "Login failed")]
    with pytest.raises(RuntimeError, match="Login command didn't generate.*"):
        executor.skopeo_login("quay_host", "quay_user", "quay_token")


@mock.patch("pubtools._quay.command_executor.LocalExecutor._run_cmd")
def test_skopeo_tag_images(mock_run_cmd):
    executor = command_executor.LocalExecutor()

    executor.tag_images("quay.io/repo/image:1", ["quay.io/repo/dest:1", "quay.io/repo/dest:2"])
    assert mock_run_cmd.call_args_list == [
        mock.call(
            "skopeo copy docker://quay.io/repo/image:1 docker://quay.io/repo/dest:1 --format v2s2"
        ),
        mock.call(
            "skopeo copy docker://quay.io/repo/image:1 docker://quay.io/repo/dest:2 --format v2s2"
        ),
    ]


@mock.patch("pubtools._quay.command_executor.LocalExecutor._run_cmd")
def test_skopeo_tag_images_all_arch(mock_run_cmd):
    executor = command_executor.LocalExecutor()

    executor.tag_images(
        "quay.io/repo/image:1", ["quay.io/repo/dest:1", "quay.io/repo/dest:2"], True
    )
    assert mock_run_cmd.call_args_list == [
        mock.call(
            "skopeo copy --all docker://quay.io/repo/image:1 docker://quay.io/repo/dest:1"
            " --format v2s2"
        ),
        mock.call(
            "skopeo copy --all docker://quay.io/repo/image:1 docker://quay.io/repo/dest:2"
            " --format v2s2"
        ),
    ]


@mock.patch("pubtools._quay.command_executor.LocalExecutor._run_cmd")
def test_skopeo_inspect(mock_run_cmd):
    mock_run_cmd.return_value = ('{"aaa":"bbb"}', "")
    executor = command_executor.LocalExecutor()

    ret = executor.skopeo_inspect("quay.io/repo/image:1")
    mock_run_cmd.assert_called_once_with("skopeo inspect docker://quay.io/repo/image:1")
    assert ret == {"aaa": "bbb"}

    ret = executor.skopeo_inspect("quay.io/repo/image:1", raw=True)
    assert ret == '{"aaa":"bbb"}'



import json
import logging
import mock
import pytest
import requests_mock
import requests
import os

from pubtools._quay import exceptions
from pubtools._quay.utils import misc
from .utils.misc import sort_dictionary_sortable_values, compare_logs, mock_entry_point

# flake8: noqa: E501

LOG = logging.getLogger()
logging.basicConfig()
LOG.setLevel(logging.INFO)


def test_run_entrypoint(caplog):
    def side_effect_entrypoint(*args):
        LOG.warning("log generated by entrypoint")
        LOG.info("environment variable: {0}".format(os.environ["PYXIS_PASSWORD"]))
        return {"pyxis": "data"}

    with mock_entry_point(
        "pubtools-pyxis", "console_scripts", "pubtools-pyxis-get-operator-indices"
    ) as entrypoint:
        entrypoint.side_effect = side_effect_entrypoint
        ret_val = misc.run_entrypoint(
            ("pubtools-pyxis", "console_scripts", "pubtools-pyxis-get-operator-indices"),
            "pubtools-pyxis-get-operator-indices",
            [
                "--pyxis-krb-principal",
                "some-principal@REDHAT.COM",
                "--organization",
                "redhat-operators",
                "--ocp-versions-range",
                "v4.5",
                "--pyxis-krb-ktfile",
                "/etc/pub/some.keytab",
            ],
            {"PYXIS_PASSWORD": "some-password"},
        )
        expected_logs = [
            "Running task with arguments:",
            "pubtools-pyxis-get-operator-indices --pyxis-krb-principal some-principal@REDHAT\.COM --organization \\\\",
            "redhat-operators --ocp-versions-range v4.5 --pyxis-krb-ktfile /etc/pub/some\.keytab",
            "log generated by entrypoint",
            "environment variable: some-password",
        ]
        compare_logs(caplog, expected_logs)


def test_run_entrypoint_mod(caplog):
    def side_effect_entrypoint(*args):
        LOG.warning("log generated by entrypoint")
        LOG.info("environment variable: {0}".format(os.environ["PYXIS_PASSWORD"]))
        return {"return": "value"}

    with mock_entry_point(
        "pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"
    ) as entrypoint:
        entrypoint.side_effect = side_effect_entrypoint
        ret_val = misc.run_entrypoint(
            ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            None,
            ["some-ep-argument"],
            {"PYXIS_PASSWORD": "some-password"},
        )
        expected_logs = [
            "Running task with arguments:",
            "pubtools-sign-cosign-signature-list some-ep-argument",
            "log generated by entrypoint",
            "environment variable: some-password",
        ]
        compare_logs(caplog, expected_logs)


def test_run_entrypoint_without_args(caplog):
    def side_effect_entrypoint(*args):
        LOG.warning("log generated by entrypoint")
        LOG.info("environment variable: {0}".format(os.environ["PYXIS_PASSWORD"]))
        return {"pyxis": "data"}

    with mock_entry_point(
        "pubtools-pyxis", "console_scripts", "pubtools-pyxis-get-operator-indices"
    ) as entrypoint:
        entrypoint.side_effect = side_effect_entrypoint
        ret_val = misc.run_entrypoint(
            ("pubtools-pyxis", "console_scripts", "pubtools-pyxis-get-operator-indices"),
            "pubtools-pyxis-get-operator-indices",
            [],
            {"PYXIS_PASSWORD": "some-password"},
        )
        expected_logs = [
            "Running task with arguments:",
            "pubtools-pyxis-get-operator-indices",
            "log generated by entrypoint",
            "environment variable: some-password",
        ]
        compare_logs(caplog, expected_logs)


def test_run_entrypoint_not_capture_stdout(capsys):
    def side_effect_entrypoint(*args):
        print("print generated by entrypoint")

    with mock_entry_point(
        "pubtools-pyxis", "console_scripts", "pubtools-pyxis-upload-signatures"
    ) as entrypoint:
        entrypoint.side_effect = side_effect_entrypoint
        misc.run_entrypoint(
            ("pubtools-pyxis", "console_scripts", "pubtools-pyxis-upload-signatures"),
            "pubtools-pyxis-upload-signature",
            [],
            {},
            False,
        )
        captured = capsys.readouterr()
        assert captured.out == "print generated by entrypoint\n"


def test_get_internal_repo_name():
    internal_name = misc.get_internal_container_repo_name("namespace/repo")
    assert internal_name == "namespace----repo"


def test_get_internal_repo_name_no_delimeter():
    internal_name = misc.get_internal_container_repo_name("namespace-repo")
    assert internal_name == "namespace-repo"


def test_get_internal_repo_name_errors():
    with pytest.raises(ValueError, match="Input repository containing a delimeter.*"):
        misc.get_internal_container_repo_name("/namespacerepo")

    with pytest.raises(ValueError, match="Input repository containing a delimeter.*"):
        misc.get_internal_container_repo_name("namespacerepo/")

    with pytest.raises(ValueError, match="Input repository containing a delimeter.*"):
        misc.get_internal_container_repo_name("name/space/repo")


def test_get_external_repo_name():
    internal_name = misc.get_external_container_repo_name("namespace----repo")
    assert internal_name == "namespace/repo"


def test_get_external_repo_name_no_delimeter():
    internal_name = misc.get_external_container_repo_name("namespace-repo")
    assert internal_name == "namespace-repo"


def test_get_external_repo_name_errors():
    with pytest.raises(ValueError, match="Input repository containing a delimeter.*"):
        misc.get_external_container_repo_name("----namespacerepo")

    with pytest.raises(ValueError, match="Input repository containing a delimeter.*"):
        misc.get_external_container_repo_name("namespacerepo----")

    with pytest.raises(ValueError, match="Input repository containing a delimeter.*"):
        misc.get_external_container_repo_name("name----space----repo")


def test_get_pyxis_ssl_paths_hook(fake_cert_key_paths):
    cert, key = misc.get_pyxis_ssl_paths({"pyxis_server": "some-url"})
    assert cert == "/path/to/file.crt"
    assert key == "/path/to/file.key"


def test_get_pyxis_ssl_paths_target_settings():
    cert, key = misc.get_pyxis_ssl_paths(
        {
            "pyxis_server": "some-url",
            "pyxis_ssl_cert": "/some/file.crt",
            "pyxis_ssl_key": "/some/file.key",
        }
    )
    assert cert == "/some/file.crt"
    assert key == "/some/file.key"


def test_get_pyxis_ssl_paths_error():
    with pytest.raises(ValueError, match="No key and certificate paths were.*"):
        cert, key = misc.get_pyxis_ssl_paths({"pyxis_server": "some-url"})


@mock.patch("time.sleep")
@mock.patch("pubtools._quay.utils.misc.timestamp")
def test_run_with_retries(mock_timestamp, mock_sleep):
    mock_timestamp.side_effect = [ValueError, 10]
    misc.run_with_retries(misc.timestamp, "Get timestamp")

    assert mock_timestamp.call_count == 2
    mock_sleep.assert_called_once_with(0)


@mock.patch("time.sleep")
@mock.patch("pubtools._quay.utils.misc.timestamp")
def test_run_with_retries_fail(mock_timestamp, mock_sleep):
    mock_timestamp.side_effect = [ValueError] * 4
    with pytest.raises(ValueError):
        misc.run_with_retries(misc.timestamp, "Get timestamp")

    assert mock_timestamp.call_count == 4
    assert mock_sleep.call_count == 3
    assert mock_sleep.call_args_list[0] == mock.call(0)
    assert mock_sleep.call_args_list[1] == mock.call(10)
    assert mock_sleep.call_args_list[2] == mock.call(20)


@mock.patch("time.sleep")
@mock.patch("pubtools._quay.utils.misc.timestamp")
def test_retry(mock_timestamp, mock_sleep):
    mock_timestamp.side_effect = [1, 2, 3, 4]

    @misc.retry("Add value to timestamp")
    def add_value_to_timestamp(value):
        timestamp = misc.timestamp()
        if isinstance(value, str):
            raise ValueError("Not a number!")
        return timestamp + value

    with pytest.raises(ValueError):
        add_value_to_timestamp("not a number")

    assert mock_timestamp.call_count == 4
    assert mock_sleep.call_count == 3
    assert mock_sleep.call_args_list[0] == mock.call(0)
    assert mock_sleep.call_args_list[1] == mock.call(10)
    assert mock_sleep.call_args_list[2] == mock.call(20)


@mock.patch(
    "builtins.open",
    new_callable=mock.mock_open,
    read_data='{"auths": {"registry": {"auth": "dXNlcjpwYXNz"}}}',
)
@mock.patch("os.path.isfile")
def test_get_basic_auth(mock_isfile, mock_file):
    mock_isfile.return_value = True
    assert misc.get_basic_auth("registry") == ["user", "pass"]


@mock.patch("pubtools._quay.utils.misc.run_entrypoint")
def test_get_repo_metadata(
    mock_run_entrypoint,
    target_settings,
    container_multiarch_push_item,
    operator_push_item_ok,
    fake_cert_key_paths,
):
    hub = mock.MagicMock()
    mock_run_entrypoint.return_value = {"key": "value"}
    res = misc.pyxis_get_repo_metadata("some_repo", target_settings)

    assert res == {"key": "value"}
    mock_run_entrypoint.assert_called_once_with(
        ("pubtools-pyxis", "console_scripts", "pubtools-pyxis-get-repo-metadata"),
        "pubtools-pyxis-get-repo-metadata",
        [
            "--pyxis-server",
            "pyxis-url.com",
            "--pyxis-ssl-crtfile",
            "/path/to/file.crt",
            "--pyxis-ssl-keyfile",
            "/path/to/file.key",
            "--repo-name",
            "some_repo",
        ],
        {},
    )


def test_set_aws_kms_environment_variables_missing_setting(target_settings, caplog):
    caplog.set_level(logging.WARNING)
    misc.set_aws_kms_environment_variables(target_settings, "test_profile")
    expected_logs = [
        "Target settings are missing the aws_kms_credentials property, "
        "cannot set AWS KMS environment variables"
    ]
    compare_logs(caplog, expected_logs)


def test_set_aws_kms_environment_variables_missing_profile(target_settings, caplog):
    caplog.set_level(logging.WARNING)
    target_settings["aws_kms_credentials"] = {
        "other_profile": {
            "aws_access_key_id": "id1",
            "aws_secret_access_key": "key1",
            "aws_default_region": "us-east-1",
        }
    }
    misc.set_aws_kms_environment_variables(target_settings, "test_profile")
    expected_logs = ["AWS KMS profile test_profile not found in the target settings"]
    compare_logs(caplog, expected_logs)


def test_set_aws_kms_environment_variables_missing_variables(target_settings, caplog):
    caplog.set_level(logging.WARNING)
    target_settings["aws_kms_credentials"] = {"test_profile": {}}
    misc.set_aws_kms_environment_variables(target_settings, "test_profile")
    expected_logs = [
        "Cannot set AWS KMS environment variable AWS_ACCESS_KEY_ID, "
        "value missing in profile test_profile",
        "Cannot set AWS KMS environment variable AWS_SECRET_ACCESS_KEY, "
        "value missing in profile test_profile",
        "Cannot set AWS KMS environment variable AWS_DEFAULT_REGION, "
        "value missing in profile test_profile",
    ]
    compare_logs(caplog, expected_logs)

    assert "AWS_ACCESS_KEY_ID" not in os.environ
    assert "AWS_SECRET_ACCESS_KEY" not in os.environ
    assert "AWS_DEFAULT_REGION" not in os.environ


def test_set_aws_kms_environment_variables(target_settings, caplog):
    caplog.set_level(logging.WARNING)
    target_settings["aws_kms_credentials"] = {
        "test_profile": {
            "aws_access_key_id": "id1",
            "aws_secret_access_key": "key1",
            "aws_default_region": "us-east-1",
        }
    }
    misc.set_aws_kms_environment_variables(target_settings, "test_profile")
    expected_logs = []
    compare_logs(caplog, expected_logs)

    assert os.environ["AWS_ACCESS_KEY_ID"] == "id1"
    assert os.environ["AWS_SECRET_ACCESS_KEY"] == "key1"
    assert os.environ["AWS_DEFAULT_REGION"] == "us-east-1"
